version = "$VERSION 1.05 {06/19/2017 10:11:08 AM} VERSION$"
/*
   File...: BDEALIAS.CC
   Author.: Romain Strieff
   Date...: 09/21/1998 (turned into a .CC by Ken Mayer)
            02/10/1999 added databaseFound method
            02/12/1999 moved BDECFG_FILE(), localShare(), and
                       fromUnicode() from MISCAPI.PRG to here.
            March 28, 1999 --
                       Fixed some minor things and added
                       the cTableFormat parameter to the 
                       createAlias() method.
            March 29, 1999 -- 
                       Gary fixed databaseFound a bit,
                       added databaseDir()
            January 29, 2002 --
                       Fred (Meuler?) from the dBASE
                       Newsgroups found a fix for
                       the localShare method's return
                       statement (a bug was there). 
            April 17, 2002 --
                       KM -- added ListAliases method,
                       which uses the databases getSchema,
                       and returns a one or two column array
                       of database aliases (and their paths).
            April 26, 2002 --
                       KM -- added ListTables method,
                       which uses database getSchema, and
                       returns a single-column array
                       of the table names in the alias
                       provided.
            September 12, 2002 --
                       Added ListViews and ListProcedures methods,
                       which can be used in some SQL Database 
                       servers to list any Views or Stored Procedures
                       in the alias.
             06/28/2011 -- REM Changes for DeleteAlias to delete a 
                       persistant alias. And other minor changes. 
                       
                       Important: The location of the IDAPI.CFG file
                       should be in the ProgramData folder,
                       rather than in "Program Files". 
                       
                       Here:
                       C:\ProgramData\Common Files\Borland\BDE
                       
                       NOT here:
                       C:\Program Files (x86)\Common Files\Borland\BDE
                       
                       When you install dBASE look at where the BDE 
                       gets installed, and make sure it goes to the ProgramData
                       folder. This is the current folder under the
                       most recent versions of dBASE (11, possibly
                       dBASE Plus 10 or earlier ...).
                       
                       According to Marty Kay (R&D at dBASE), you can
                       check the registry settings for the path to the
                       IDAPI.CFG file:
                       
                       32-bit Windows:
                          HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine 
                       64-bit Windows:
                          HKEY_LOCAL_MACHINE\SOFTWARE\WOW6432Node\Borland\Database Engine 
                          
                       The value name:
                           CONFIGFILE01
                       The Type: 
                           REG_SZ
                       The value will be the path to the IDAPI.CFG file.
                       
                       The DeleteAlias() method was tested by Rich under dBASE Plus 
                       2.8 and dBASE Plus 11.3.1, on Windows 7 , Windows 10 ...

   This allows you to delete or create a BDE Alias without
   having to exit dBASE and run the BDE Administrator
   application. This uses the BDE API ...

   Useful for deployment of an application ...

   Methods included here:

      databaseFound() -- returns logical value based on whether
        or not the BDE Alias you are looking for is set in your
        installation of the BDE or not. Can be used to determine
        if you should create it or not ...
      createAlias() -- creates a BDE Alias
      deleteAlias() -- deletes a BDE Alias
      BDECFG_File() -- returns the name of the BDE configuration
                       file "in use" by the BDE at that time.
      localShare()  -- returns a logical based on the state
                       of this setting in the BDE.
      DatabaseDir() -- returns the directory of an alias supplied
                       as a parameter
      ListAliases() -- returns an array of BDE Alias names and
                       their paths.
      ListTables( cAlias ) -- returns array of tables in a
                       specific BDE Alias
      ListViews( cAlias ) -- returns array of Views (some databases
                       allow these) in a specific BDE Alias -- if
                       array.size is zero, there are no views.
      ListProcedures( cAlias ) -- returns array of procedures
                       in a specific alias (some databases 
                       allow stored procedures) -- if array.size is
                       zero, there are no procedures.

   Examples:

      set procedure to bdealias.cc additive
      b = new BDEAlias()
      ? b.databaseFound( "MYALIAS" )
      ? b.DeleteAlias( "MYALIAS" )
      ? b.CreateAlias( "MYALIAS", "C:\TEST" )
      ? b.BDECFG_File()
      ? b.localShare()
      ? b.databaseDir( "MYALIAS" )
      aAliases = b.ListAliases( [<lPaths>] )
      i.e.,
      aAliases = b.ListAliases()
        // returns a single column array of alias names
        // or:
      aAliases = b.ListAliases( true )
        // returns a two-column array of names and paths
      ? aAliases[1,1] // Alias Name
      ? aAliases[1,2] // Alias path
        // or alternatively
        for i = 1 to aAliases.size/2
            ? aAliases[i,1], aAliases[i,2]
        next
      aTables = b.ListTables( "MYALIAS" )
        for i = 1 to aTables.size
            ? aTables[ i ]
        next

      // -------------------------------------------------
      // example of creating an InterBase alias in detail:
      cIPath = "c:\MyTest;"+;
               "SERVER NAME:c:\MyTest\testdata.gdb;"+;
               "USER NAME:SYSDBA;"+;
               "LANGDRIVER:'ascii' ANSI;"+;
               "SQLQRYMODE:SERVER;"+;
               "SQLPASSTHRU MODE:SHARED NOAUTOCOMMIT"
     b.createAlias( "MYINTERBASE", cIPath, "INTRBASE" )
*/

class BDEALias

  function dataBaseFound( cDatabase )
      local dbArray, lFound, i
      dbArray = new database().getschema( "Databases" )
      lFound = false
      for i = 1 to dbArray.size
         if upper( cDatabase ) == upper( dbArray[i] )
            lFound := true
            exit
         endif
      endfor
   return lFound

   function DatabaseDir
      parameters cDatabase
      local oDatabase,cRet
      /*
         returns the path for the alias supplied
         as a parameter (null if it does not exist)

         Based on code posted by Romain Strieff 
         in the Visual dBASE Newsgroups

         Modified to accept a string for an alias name
         instead of a database object by Gary White
      */
      cRet = null
      if type("cDatabase") == "C" and not empty(cDatabase)
         if this.databaseFound( cDatabase )
            if type("GetAliasPath") # "FP"
               extern cint GetAliasPath(cHandle,cString,;
                  cString,cString);
                  IDAPI32 from "DbiFormFullName"
            endif
            cRet=space(261)
            oDatabase = new database()
            oDatabase.databaseName := cDatabase
            oDatabase.active := true
            if ( GetAliasPath(oDatabase.handle,"DummyTableName","",cRet ) == 0 )
               cRet=cRet.left(cRet.lefttrim().lastindexof("\"))
               else
               cRet=""
            endif
            oDatabase.active := false
            release object oDatabase
         endif
      endif
   return cRet

  function OLDDeleteAlias(cAliasName)
     //Deletes an existing Alias
      if type("DbiDeleteAlias") # "FP"
         extern cint DbiDeleteAlias(;
                cHandle,;
                CPTR ) idapi32
      endif

    return DbiDeleteAlias(null,this.toUnicode(cAliasName))==0

   **************************************************************************   
	function DeleteAlias
   **************************************************************************   
      parameters cDatabase
      local orgCfgMode2,nCfgMode2
                                   
   	#define  sesCFGMODE2    0x2000c // rw CFGMode, Mode of configuration file.

      #define 	cfgmNone			0x0
      #define 	cfgmVirtual		0x1
      #define  cfgmPersistent 0x2   // Persistent only
      #define  cfgmSession  	0x4   // Session relative only
      #define  cfgmAll 			0x7   // All (union)

      hSession = _app.session.handle

      if type([DbiSetProp]) # [FP]
         extern cushort DbiSetProp(;
                cHandle,;
                cLong,;
                cLong ) IDAPI32
      endif

      if type([DbiGetProp]) # [FP]
         extern cushort DbiGetProp(;
         		 chandle,;
                culong,;
                cptr culong,;
                cushort,;
                cptr cushort) IDAPI32
      endif

      #define DBIERR_NONE 0
      if type([DbiDeleteAlias]) # [FP]
         extern cint DbiDeleteAlias(;
                cHandle,;
                CPTR ) IDAPI32
      endif

		// see what original sesCFGMODE2 setting is
      orgCfgMode2 = 0
      DbiGetProp(hSession,sesCFGMODE2,orgCfgMode2,null,null)
      //? "Original cfgmNone: " + (orgCfgMode2 == cfgmNone)

      // change sesCFGMODE2 to cfgmPersistent
      DbiSetProp(hSession,sesCFGMODE2,cfgmPersistent)
		nCfgMode2 = 0
      DbiGetProp(hSession,sesCFGMODE2,nCfgMode2,null,null)
      ? "Changed to cfgmPersistent: " + (nCfgMode2 == cfgmPersistent)

      // delete alias
      lSuccess = DbiDeleteAlias(NULL,class::toUnicode(cDatabase))==DBIERR_NONE

 		// see if we set sesCFGMODE2 back to where it was
      DbiSetProp(hSession,sesCFGMODE2,orgCfgMode2)
      nCfgMode2 = 0
      DbiGetProp(hSession,sesCFGMODE2,nCfgMode2,null,null)
      ? "Changed back to original: " + (nCfgMode2 == orgCfgMode2)

   return lSuccess

  function CreateAlias(cAliasName, cPath, cTableFormat)
  // cAliasName = the Alias name that the database is to
  //              be referred to
  // cPath      = the full path to the database
  // cTableFormat = the table format (i.e., INTRBASE, DBASE, etc.)

     if pCount() < 3
        cTableFormat = "DBASE"
     endif
     //Creates an Alias

     if type("DbiAddAlias") # "FP"
        extern cint DbiAddAlias(;
               cHandle,;
               CPTR,;
               CPTR,;
               CPTR,;
               clogical) idapi32
     endif
  return DbiAddAlias(null,;
                     this.toUnicode(cAliasName),;
                     this.toUnicode(cTableFormat),;
                     this.toUnicode("PATH:" + cPath),;
                     TRUE)==0

   function BDECFG_File
      local cBuffer,cRet
      /*
         BDECFG_File -- returns the name of the currently
         used BDE Configuration file. (Uses BDE API)
         Note: this uses fromUnicode() below ...

         Posted by Romain Strieff in the Visual dBASE Newsgroups
         February 2, 1999
      */

      if type( "DbiGetSysConfig" ) # "FP"
         extern cInt DbiGetSysConfig( cPtr ) IDAPI32
      endif
      cBuffer = space( 256 )
      DbiGetSysConfig( cBuffer )
      // minor code fix noted by Lane Coddington
      cRet = substr( this.fromUnicode( cBuffer ), 54, 128 )
      cRet = left( cRet, at(chr(0), cRet ) - 1 )
   return cRet

   function localShare
      local cBuffer
      /*
         localShare -- returns a logical true/false based on
         the localShare setting of the BDE. 
         Note: this uses fromUnicode() below ...

         Posted by Romain Strieff in the Visual dBASE Newsgroups
         February 2, 1999
      */

      if type( "DbiGetSysConfig" ) # "FP"
         extern cInt DbiGetSysConfig( cPTR ) idapi32
      endif

      cBuffer = space( 256 )
      DbiGetSysConfig( cBuffer )

   return asc( left( this.fromUnicode( cBuffer ), 1 ) ) == 1

   function fromUnicode( c )
      /*
         fromUnicode returns standard characters rather than
         the Unicode character set (which is larger).

         Posted by Romain Strieff in the Visual dBASE Newsgroups
         February 2, 1999
      */
      local cText, nLen, nInd
      nLen = len( c )
      cText = ""
      for nInd = 0 to nLen - 1
          cText += chr( c.getByte( nInd ) )
      endfor
		return cText

   function ToUnicode(c)
	   // Converts to Unicode
      local cTemp, x
      cTemp = Replicate(Chr(0), ((Len(c) + 1) / 2) + ;
                                ((Len(c) + 1) % 2))
      For x = 1 To Len(c)
         cTemp.SetByte(x - 1, Asc(SubStr(c, x)))
      EndFor
	   return cTemp

   function ListAliases
      /*
          This method returns a two-column array of
          BDE Aliases and their paths, which can be used
          by the developer to do whatever is needed
          from there. 

          Ken Mayer, April 17, 2002

          The lPaths parameter will return the second
          column with the path. Without it, you get
          a single column array of just the names ...
      */
      parameter lPaths
     	local aSchema,i
      // Get the names of the aliases:
      aSchema = _app.databases.current.getSchema("Databases")

      if pCount() > 0 and lPaths == true
         // add a column, and leave data where it is
         aSchema.resize( aSchema.size, 2, 1 )
         for i = 1 to (aSchema.size/2)
             aSchema[ i, 2 ] = this.DatabaseDir( aSchema[ i, 1 ] )
         next
      endif

   return aSchema

   function ListTables
      /*
          This method returns a single-column array of
          Tables in a BDE Aliases which can be used
          by the developer to do whatever is needed
          from there. 
      */
      parameter cAlias
      local aSchema
		aSchema = new array()
      try
          aSchema = new database(cAlias).getSchema( "Tables" )
      catch( Exception E )
          msgbox( "Error: "+e.message, "BDEAlias::Error", 16 )
      endtry
   	return aSchema

   function ListViews
      /*
          This method returns a single-column array of
          Views in a BDE Alias which can be used
          by the developer to do whatever is needed
          from there. 

      */
      parameter cAlias
      local aSchema
		aSchema = new array()
      try
	      aSchema = new database( cAlias ).getSchema( "VIEWS" )
      catch( Exception E )
          msgbox( "Error: "+e.message, "BDEAlias::Error", 16 )
		endtry
	   return aSchema

   function ListProcedures
      /*
          This method returns a single-column array of
          Stored Procedures in a BDE Alias which can be used
          by the developer to do whatever is needed
          from there. 
      */
      parameter cAlias
      local aSchema
		aSchema = new array()
      try
	      aSchema = new database( cAlias ).getSchema( "PROCEDURES" )
      catch( Exception E )
          msgbox( "Error: "+e.message, "BDEAlias::Error", 16 )
		endtry
	   return aSchema
endclass

/*
    End of File: BDEALIAS.CC
*/

