/*
   BDETempFolder.prg
   A small set of functions used to create a new BDE temporary,
   tell the BDE to use that for some work, and then on a second 
   call, to delete the path created, and reset the BDE to use
   its default temp setting.

   Created by Rich Muller (www.autotraker.com). 
   Cleaned up only a tiny amount by Ken Mayer for the dUFLP.

   Usage:
       set procedure to BDETempFolder additive
       SetBDETempDir()
       // do whatever
       SetBDETempDir()

   You can also use the MakeDirTree() function independently of the
   SetBDETempDir() function:

      ? MakeDirTree( "C:\SomePath\More\And even More" )
      // Returns true if successful, false if not.
*/

/* 
   SetBDETempDir()
   Rich Muller - www.autotraker.com
   Sets and then deletes a temp directory for use by the BDE.

   The first call to this function sets the temp directory and
   the second returns it back to default.

   App variable _app.BDETemp holds the directory path and is
   set to NULL when we are set back to default.

   Based partly on code from the dBase newsgroup

   Usage: SetBDETempDir() // first call sets it, second call returns 
                          // to default deleting the temporary folder
                          // created in the first place ...
*/
function SetBDETempDir
   if type("_app.BDETemp")=="U"
      _app.BDETemp = fUnique()
      makeDirTree(_app.BDETemp)
   else
      cDir = _app.BDETemp
      _app.BDETemp = NULL
   endif

   #include "vdbase.h"
   #define DBIMAXMSGLEN 127
   if type("DbiSetPrivateDir")<>"FP"
      extern cushort DbiSetPrivateDir(cstring) idapi32
   endif
   if type("DbiSetDirectory")<>"FP"
      extern cushort DbiSetDirectory(culong, cstring) idapi32.dll
   endif
   if type("DbiGetErrorString")<>"FP"
      extern cushort DbiGetErrorString(cword, cstring) idapi32.dll
   endif

   // passing NULL to DbiSetPrivateDir reset to the default
   // startup directory. DbiSetPrivateDir should be set to a
   // exclusive directory
   lResult1 = IsDbiOk(DbiSetPrivateDir(_app.BDETemp))
   lResult2 = IsDbiOk(DbiSetDirectory(;
    _app.databases.current.handle,setto("Directory")))

   if _app.BDETemp == NULL
    try
         aFiles = new array()
         nFiles = aFiles.dir(cDir+"\*.*","HS")
         for nFile = 1 to nFiles
          new file().delete(cDir+"\"+aFiles[ nFile,1 ])
         endfor
         if type("RemoveDirectoryA") # "FP"
            extern cLogical RemoveDirectoryA( cString ) kernel32
         endif
         RemoveDirectoryA(cDir)
    catch(exception e)
       msgbox('Error '+e.code+': '+e.message, 'Error!',16)
      endtry
   endif

   if lResult1 and lResult2
    return true
   endif
return false

/*
   IsDbiOK()
   Rich Muller -- www.autotraker.com
   Returns the error string in a msgbox() call of a BDE
   error. Pass the error number to the function ...
*/
function IsDbiOk(iResult)

   // Returns BDE Error string
   local cErrMsg, lResult, c
   lResult = (iResult == 0)
   if not lResult
      cErrMsg = space(DBIMAXMSGLEN+1)
      if type("DbiGetErrorString")<>"FP"
         extern cushort DbiGetErrorString(cword, cstring) idapi32.dll
      endif
      DbiGetErrorString(iResult, cErrMsg)
      msgbox('Error '+ltrim(str(iResult))+': '+cErrMsg, 'BDE Error!',16)
      return false
   endif
return true

/*
   MakeDirTree()
   Rich Muller - www.autotraker.com
   This function will build an entire directory tree
   just pass the entire tree and it will build it.

      cDirTree = "c:\temp2\temp2\temp3\temp4\temp5\temp6"
      MakeDirTree(cDirTree)

   Returns true if the directory exists if it has build it.

   Can use either imageHlp.dll or dbgHelp.dll (system32 directory).
   Since some versions of imageHelp.dll may not support this function,
   we are putting it in a try/catch/etc.
*/
function MakeDirTree
   parameters cDirTree
   local cDir
   cDir = iif(right(cDirTree,1)=="\",cDirTree,cDirTree+"\")
   if type("MakeSureDirectoryPathExists")<>"FP"
      try
         extern cLogical MakeSureDirectoryPathExists(cString) "IMAGEHLP.DLL"
      catch(exception e)
      endtry
   endif
   if type("MakeSureDirectoryPathExists")<>"FP"
      extern cLogical MakeSureDirectoryPathExists(cString) "dbgHelp.dll"
   endif
return MakeSureDirectoryPathExists(cDir)
