
/*
   ----------------------------------------------------------------
    Description:
 
       This custom entryfield (and spinbox) includes a pushbutton 
       that can be used to calls a popup calendar. The user can 
       pick a date from the calendar and it will be returned to 
       the entryfield (or spinbox).
 
    Programmers: 
 
       Michael Nuwer
 
    History: 
       12/27/2003 -- Spinbox control added.
       09/28/2003 -- First version of this custom class file.

    Instantiation: 
 
       set procedure to DatePicker.cc additive
       create/modify a form, place an instance of this 
       custom entryfield on the form as a normal entryfield 
       
       Form metric must be 6 - pixels

    Dependencies:

       Calendar.cc from the dBLClasses\formControls folder
       VStruct.prg by Ken Chan
       VStruct.h
       Calendar.bmp

*/
class efDATEPICKER(parentObj) of ENTRYFIELD(parentObj) custom
   with (this)
      onOpen = class::DATEPICKER_ONOPEN
      height = 22
      left = 4
      top = 1
      width = 154
      metric = 6	
      value = {  /  /  }
   endwith

   function DATEPICKER_onOpen
      this.formatButton()
      return

   function formatButton
      privat cBtnName, c, cn  //, click

      //  the button name is PBefDatePickerX....

      cBtnName = 'PB'+trim(This.Name)
      c = 'this.parent.'+cBtnName
      cn = c+' = new PUSHBUTTON(this.parent)'
      &cn
      cn = c+'.owner = this'
      &cn
      with (&c)
         group    = true
         Top      = this.top + 2
         Left     = this.left+this.width - 24
         Height   = this.height - 3
         Width    = 22  
         Text     = ""
         OnClick  = CLASS::PBDatePicker_OnClick
         PageNo   = this.PageNo
         UpBitmap = "filename:2 Calendar.bmp"
         tabStop  = false
      endwith
      return

   function PBDatePicker_OnClick
      private oChild  // so we can use TYPE()
      oChild = new calendarForm()
      oChild.mdi = false

      if this.parent.metric = 6
         //controls form is in pixels - position calculator near spinbox
         if type("GetWindowRect") # "FP"
            extern CLOGICAL GetWindowRect(CHANDLE, CPTR) User32
         endif

         set procedure to :dUFLP:vStruct.prg additive
         oRect = new VStruct( 8*2 )
         GetWindowRect(this.parent.hwnd, oRect )
         
         parentLeft   = oRect.getLong( 0, false )
         parentTop    = oRect.getLong( 4, false )
         parentWidth  = oRect.getLong( 8, false )
         parentHeight = oRect.getLong( 12, false )
         close procedure :dUFLP:vStruct.prg

         if this.parent.baseClassName == "FORM"
            nTitleBar = 20
         else
            nTitleBar = 0
         endif 
         oChild.top  = parentTop + this.owner.top + this.owner.height ;
                         + nTitleBar + 3
         oChild.left = parentLeft + this.owner.left + this.owner.width ;
                         - oChild.width - 3
         oChild.autoCenter = false
      endif

      //  Pass the current value if there is one.
      if not empty(this.owner.value)
         oChild.date = this.owner.value
      endif
      oChild.readModal()

      if type("oChild.date") == "D" 
         // The dB2K Calendar class creates
         // a date
         this.owner.value = oChild.date
         this.owner.setFocus()
      elseif type("oChild.date") == "O"
         // The dBASE Plus Calendar class creates
         // a date object
         this.owner.value = dttod(oChild.date)
         this.owner.setFocus()
      endif
      oChild.release()
      return
endclass

class sbDATEPICKER(parentObj) of SPINBOX(parentObj) custom
   with (this)
      onOpen = class::DATEPICKER_ONOPEN
      height = 22
      left = 4
      top = 1
      width = 154
      metric = 6	
      value = {  /  /  }
   endwith

   function DATEPICKER_onOpen
      this.formatButton()
      return

   function formatButton
      privat cBtnName, c, cn  //, click

      //  the button name is PBDatePickerX....

      cBtnName = 'PB'+trim(This.Name)
      c = 'this.parent.'+cBtnName
      cn = c+' = new PUSHBUTTON(this.parent)'
      &cn
      cn = c+'.owner = this'
      &cn
      with (&c)
         group    = true
         Top      = this.top + 2
         Left     = this.left+this.width - 42
         Height   = this.height - 3
         Width    = 22  
         Text     = ""
         OnClick  = CLASS::PBDatePicker_OnClick
         PageNo   = this.PageNo
         UpBitmap = "filename:2 Calendar.bmp"
         tabStop  = false
      endwith
      return

   function PBDatePicker_OnClick
      private oChild  // so we can use TYPE()
      oChild = new calendarForm()
      oChild.mdi = false

      if this.parent.metric = 6
         //controls form is in pixels - position calculator near spinbox
         if type("GetWindowRect") # "FP"
            extern CLOGICAL GetWindowRect(CHANDLE, CPTR) User32
         endif

         set procedure to :dUFLP:vStruct.prg additive
         oRect = new VStruct( 8*2 )
         GetWindowRect(this.parent.hwnd, oRect )
         
         parentLeft   = oRect.getLong( 0, false )
         parentTop    = oRect.getLong( 4, false )
         parentWidth  = oRect.getLong( 8, false )
         parentHeight = oRect.getLong( 12, false )
         close procedure :dUFLP:vStruct.prg

         if this.parent.baseClassName == "FORM"
            nTitleBar = 20
         else  // container or notebook
            nTitleBar = 0
         endif 
         oChild.top  = parentTop + this.owner.top + this.owner.height ;
                         + nTitleBar + 3
         oChild.left = parentLeft + this.owner.left + this.owner.width ;
                         - oChild.width - 3
         oChild.autoCenter = false
      endif
      if not empty(this.owner.value)
         oChild.date = this.owner.value
      endif
      oChild.readModal()
      if type("oChild.date") == "D" 
         // The dB2K Calendar class creates
         // a date
         this.owner.value = oChild.date
         this.owner.setFocus()
      elseif type("oChild.date") == "O"
         // The dBASE Plus Calendar class creates
         // a date object
         this.owner.value = dttod(oChild.date)
         this.owner.setFocus()
      endif
      oChild.release()
      return
endclass

class calendarForm of FORM
   set procedure to :formControls:Calendar.cc additive
   with (this)
      onOpen = class::FORM_ONOPEN
      metric = 6	// Pixels
      height = 158
      left = 268
      top = 20
      width = 177
      text = "Date Selector"
      autoCenter = true
      mdi = false
      smallTitle = true
      sizeable = false
      escExit = false
      maximize = false
      minimize = false
   endwith

   this.CALENDAR1 = new CALENDAR(this)
   with (this.CALENDAR1)
      onOpen = class::calendar_onopen
      left = 0
      top = 0
      width = 176
      height = 158
   endwith

   function calendar_onopen
      // override the super class method
      if type('this.CalDate') # 'D'
         this.CalDate = new date()
      endif
      if type( "form.date" ) # "U"
         this.setDate(form.date)
      else
         this.setDate(Date())      
      endif
      return

   function form_onOpen
      // register form controls with Calendar
      // Register dateFields with calendar
      this.calendar1.registerOwners(form)
      return

   function setDate(dDate)
      // set defaults for missing params
      dDate = iif(empty(dDate),date(),dDate)
      this.date = dDate      // Selected date
      this.close()      
      return

endclass


