/*
    Program...: REP2FILE.PRG
    Author....: Ken Mayer
    Date......: 09/18/1998

    The purpose of this program is to give an example
    of generating a report to file. It is not a do-all
    or be-all report-to-file generator ...

    This example requires that the MUGS sample that
    ships with Visual dBASE 7/7.01 is installed
    to the default "..\Visual dBASE\CUSTOM\MUGS\TABLES" 
    directory ...

-------------------------------------------------------------
SPECIAL NOTE: When you run this, it will ask about the links
to other tables used in the referential integrity setup
for the CUSTOMER table -- it will do this six times (!!) --
use the "OPEN READ ONLY" radio button (default), and then 
click "OK" each time -- DO NOT use the "REMOVE LINKS" option, 
or other samples in the samples directory may not function
properly!

If I were to use the datamodule for this instead, it would
work better, but datamodules in programs get a bit weirder ...
and this program is being run in a different directory ...
-------------------------------------------------------------

    Problem: The report designer in VdBASE 7.x does not
    create an ASCII text file as a reporting option
    when rendering a report.

    Explanation: The report designer is assuming WINDOWS
    capabilities, which includes proportionally spaced
    fonts, graphics, and so on, which do NOT translate well
    to ASCII, Monospaced output files.

    *****
    This is really meant to be a skeleton that can be used
    by the developer -- copy it, make appropriate changes,
    etc. 

    I cannot possibly anticipate every single situation
    you may need this for. This is here to generate ideas,
    and get you thinking in the directions you may need.

    I have included the following functions:

    Header()      // display at top of report --
                  // you can modify this to display
                  // at the top of individual "pages",
                  // but why? This is going to a single
                  // output FILE.
    Footer()      // Report footer -- useful to
                  // output totals and such.
    GroupBreak()  // Call this if you need to process
                  // a group break (i.e., a grouped
                  // report).
    SingleLine( n ) // generates a single-line of dashes
                    // n characters wide
    DoubleLine( n ) // generates a line of equal signs ( = )
                    // to give a "double" line


    **** NOTE: This assumes a single table/query, but only
    **** for this example. You can set this up to handle
    **** datamodules, multiple queries, whatever ... however,
    **** to do multiple queries, you will need to handle
    **** the extra processing to loop the child table(s)
    **** yourself ...

    **** NOTE: This is *NOT* streaming output -- we are
    **** not able to use styles, etc. here, since we are
    **** outputting directly to a text file using the 
    **** file object of Visual dBASE ...

    
*/

#DEFINE REPWIDTH 80 // 80 = "portrait"; 132 = "landscape"

cFileName  = "TESTIT.TXT" // report will go to a text file ...
                          // you could use PutFile() to
                          // get a filename from the user

// table:
cTableName = _dbwinhome+"SAMPLES\MUGS\TABLES\CUSTOMER"
// Lookup tables used for lookupSQL:
cLookup1   = _dbwinhome+"SAMPLES\MUGS\TABLES\STATE"
cLookup2   = _dbwinhome+"SAMPLES\MUGS\TABLES\COUNTRY"

/*
    Create an instance of the File class -- this is how
    we are going to generate the report. We will create
    a file by the name given above, throw the header
    out to the file, and then loop through the table ...

    DO NOT CHANGE THE NAME OF THE FILE OBJECT REFERENCE, UNLESS
    YOU DO IT THROUGH A GLOBAL REPLACE -- OTHERWISE SOME OF THESE
    ROUTINES WILL NOT FUNCTION PROPERLY!!
*/
fOut = new File() 
fOut.create( cFileName )

/*
    Create an instance of a query which will use the tablename
    defined above ... we will loop through it to generate
    the report ...

    If you want to use the datamodule instead that is in the
    sample directory, you will have to do some extra work,
    which I haven't bothered to do -- it is too much work
    for a simple *sample* program ... (I tried and found
    myself fumbling too much since this is being run from
    a different directory than the samples ...)
*/
q                   = new Query()
q.sql              := "select * from '"+cTableName+"'"
q.active           := true
q.rowset.indexName := "STATE ID" // GROUP BY STATE
// set lookups:
q.rowset.fields["STATE ID"].lookupSQL   := "select * from '"+cLookup1+"'"
q.rowset.fields["COUNTRY ID"].lookupSQL := "select * from '"+cLookup2+"'"

q.rowset.first()                 // first row in table

// Set variables here:
cGroup = ""  // this must exist and needs to be blank

// here we go:
header()     // generate header 
groupBreak() // generate first "group"

// Shorten the dot references to the fields array:
f = q.rowset.fields

// Loop through the table:
do while not q.rowset.endOfSet

   // first things first -- process group breaks:
   if f[ "STATE ID" ].value # cGroup
      groupBreak()
   endif

   // --------------------------------------------
   // FIRST LINE:
   cOutPut = space(5) // indent five spaces
   // next we build an output string:
   cOutput = trim( f["FIRST NAME"].value ) + " " + ;
             trim( f["LAST NAME"].value )
   // pad with spaces:
   cOutPut += space( 42 - len( cOutPut ) )
   // Add next field we want to output on same line
   cOutPut += "Customer ID: " + f["CUSTOMER ID"].value
   // output to file:
   fOut.puts( cOutput )

   // --------------------------------------------
   // NEXT LINE:
   // (Logic: if the first street address is empty,
   //  don't print ANY of the address ... otherwise
   //  print any of those fields that are not empty)
   if not empty( f["STREET1"].value )
      cOutPut = space(5) // indent five spaces
      cOutPut += "Address: "+f["STREET1"].value 
      fOut.puts( cOutput )

      // next line:
      if not empty( f["STREET2"].value )
         cOutPut = space(14) //5 + "address: "
         cOutPut += f["STREET2"].value
         fOut.puts( cOutput )
      endif

      // -----------------------------------------
      // next line:
      if not empty( f["CITY"].value )
         cOutPut = space(14) //5 + "address: "
         // note that the reason that the full state 
         // name is printing is the lookupSQL defined
         // at the beginning of the program ...
         cOutPut += trim( f["CITY"].value ) + ", " +;
                    trim( f["STATE ID"].value ) + "  " +;
                    trim( f["POSTAL"].value )
         if not empty( f["COUNTRY ID"].value )
            // Note that the reason that a full country
            // name is being displayed in the output is
            // due to the lookupSQL statement that
            // appears at the beginning of this program ...
            cOutPut += space(4) + f["COUNTRY ID"].value
         endif
         fOut.puts( cOutput )
      endif
   endif // street1

   // --------------------------------------------
   // next line:  -- the table being used doesn't
   //                have any data in the comments
   //                fields -- you may want to add
   //                some ... for testing, I
   //                threw a memo that had about 4 
   //                lines in it into the table
   //                to make sure this routine works.
   // NOTE: To handle memos, we must revert to some old
   // DOS code: memlines() and mLine() -- luckily the
   // developers haven't felt these needed to be removed
   // from the product:
   if not empty( f["COMMENT"].value )
      fOut.puts( space(5)+"Comments: " )
      nMemoWidth = 60 // default is 50 if nothing is provided
      nLines = memlines( f["COMMENT"].value, nMemoWidth )
      for i = 1 to nLines
          fOut.puts( space(8)+mLine( f["COMMENT"].value, i, nMemoWidth ) )
      next
   endif
   
   // --------------------------------------------
   // last line is a blank one:
   fOut.puts( "" )

   // --------------------------------------------
   q.rowset.next() // don't forget this rather important
                   // statement!
enddo

// -----------------------
// display report footer:
footer()

// -----------------------
// close the output file:
fOut.close()

// --------------------------------------------------------------
// ask user if they want to view this? (only if not in runtime --
// can't call "MODIFY COMMAND" from runtime environment ... -- we
// could issue an API call or RUN() to load notepad or wordpad, but 
// I'll leave that to someone else ...
if NOT "RUNTIME" $ version(1)
   if msgbox( "View Report: TESTIT.TXT?", "View it?", 36 ) == 6
      modi comm testit.txt
   endif
else
   msgbox( "Report is in file: TESTIT.TXT" )
endif

/*
    -----------------
    End of report ...
    -----------------
*/


/*
    -----------------------------------
    Extra Methods/functions go here ...
    -----------------------------------
*/
function Header()
   /*
       The header routine will output at the top of
       the report file, whatever you want. The first
       three lines in the sample use the center()
       function (built in to dBASE) to center
       the text within the report width (REPWIDTH)
       defined at the top of the program.
   */

   fOut.puts( center( "Test Report", REPWIDTH ) )
   fOut.puts( center( "Date: "+date(), REPWIDTH ) )
   fOut.puts( center( DoubleLine( 16 ), REPWIDTH ) )
   fOut.puts( "" )

function Footer()
   /*
       The footer routine is displayed at the end
       of the report, and can be used to display
       totals, or what-have-you. This simple version
       only displays a couple of lines and 
       "* End of Report *"
   */

   fOut.puts( "" ) 
   fOut.puts( DoubleLine( REPWIDTH ) )
   fOut.puts( center( "* End of Report *", REPWIDTH ) )
   fOut.puts( DoubleLine( REPWIDTH ) )

function GroupBreak()
   /*
       The GroupBreak routine should be called everytime
       a group changes in the row-by-row processing.
       This means you need to use the routine shown
       above or something very much like it to handle
       group breaks. You can perform calculations here,
       you can print a group header (or actually you
       could print a group footer from the previous
       group if you needed to ...). 

       The sample here only does one thing - it generates
       a group heading ...
   */

   // group header:
   fOut.puts( "" ) // blank line
   fOut.puts( "State: "+ q.rowset.fields["STATE ID"].value )
   fOut.puts( SingleLine( REPWIDTH ) )

   // you must re-assign the group value, or 
   // we'll keep "breaking":
   cGroup = q.rowset.fields["STATE ID"].value

function SingleLine( nChars )
return replicate( "-", nChars )

function DoubleLine( nChars )
return replicate( "=", nChars )


/*
    -----------------------------------------------------------------
    End of file: REP2FILE.PRG
    -----------------------------------------------------------------
*/
