/*

   Clever Consulting Services structure class.

   Author:        Andrew Shimmin
   Date:          6-Feb-2011
   Version:       0.0.0.1

   Description:   Original version.

   Author:        Andrew Shimmin
   Date:          11-Mar-2011
   Version:       0.0.0.2

   Description:   Added _NOTIFYICONDATA,tagPoint and _RECT structures for TaskBarTray.cc.

                  Modified ccsStructBase::SetAnsiString() to reset the member before
                  setting the member's value.

   Author:        Andrew Shimmin
   Date:          17-Mar-2011
   Version:       0.0.0.3

   Description:   Added release() method left out in the structures:

                  _NOTIFYICONDATA, tagPoint, _RECT.

   Author:        Andrew Shimmin
   Date:          20-Mar-2011
   Version:       0.0.0.4

   Description:   Added _MEMORYSTATUSEX structure for SysInfo class GlobalMemoryStatusEx()
                  windows API call.

   Author:        Andrew Shimmin
   Date:          20-Apr-2011
   Version:       0.0.0.5

   Description:   Changed StructBase class name to ccsStructBase.

                  Added the following methods to the ccsStructBase class:

                  GetDWORDLONG() & SetDWORDLONG()
                  GetLogical() & SetLogical()

                  Standardised this class for structures for:

                  TaskBarTray.cc v1.8
                  SysInfo.cc v1.3

   Author:        Andrew Shimmin
   Date:          5-Dec-2011
   Version:       0.0.0.6

   Description:   Added ccsStructBase class methods:

                  SetULong() to set an unsigned long value in a structure.
                  SetHandle() to set a windows handle in a structure.
                  GetHandle() to get a windows handle from a structure.

                  Modified _NOTIFYICONDATA::nIcon() method to use the above to fix
                  a bug that causes a setByte() value out of range error when
                  bit 31 of the icon handle is set.

   Author:        Andrew Shimmin
   Date:          13-Dec-2011
   Version:       0.0.0.7

   Description:   Added ccsStructBase class methods:

                  GetUQuadword() to get an unsigned quadword value in a structure
                  for the _MEMORYSTAUSEX structure returned 64-bit memory members.

   Author:        Andrew Shimmin
   Date:          27-Oct-2012
   Version:       0.0.0.8

   Description:   Added class property lMethodCount to disable by default
                  class method counting on class instantiation.

                  This prevents slow class instantiation when run from a NAS box.

   Author:        Andrew Shimmin
   Date:          10-Nov-2012
   Version:       0.0.0.9

   Description:   Added class method close().

                  Fixed error in SystemInfo structure.
*/

class ccs_structures(vInstanceId) of Timer() custom

   // properties

   this.cVersion = [0.0.0.9]
   this.cVersionDate = [10-Nov-2012]

   this.lMethodCount = false                                                  // count methods
   this.nMethods = 0                                                          // number of file methods

   class::fileMethodCount()

?program(0)+[ version: ]+this.cVersion+[ date: ]+this.cVersionDate

// Method ======================================================================

   function IamLoaded()
      return

// Method ======================================================================

   function close()

      try
         release object this
      catch (exception e)
      endtry

      return

// Method ======================================================================

   function fileMethodCount(lCountMethods)

      local f,l,lCount

      f = []
      l = []
      lCount = iif(.not.empty(lCountMethods),lCountMethods,this.lMethodCount)
      this.nMethods := 0

      if ([runtime]$lower(version()) = 0).and.(lower(right(program(0),2)) == [cc])

         if lCount                                                            // count methods?

            f := new file()
            f.open(program(0),[R])

            try

               do while .not.f.eof()

                  l := f.gets()

                  if (left(l,2) == "//").and.(lower(substr(l,4,6)) == [method])
                     this.nMethods++
                  endif

               enddo

            catch (exception e)
            finally
               f.close()
            endtry

         endif

      endif

      return

// Method ======================================================================

   function getVersion()
      return (this.cVersion)

// Method ======================================================================

   function getVersionDate()
      return (this.cVersionDate)

endclass

class _GUID of ccsStructBase(16)

// Method ======================================================================
   function data1(n)
      return
// Method ======================================================================
   function data2(n)
      return
// Method ======================================================================
   function data3(n)
      return
// Method ======================================================================
   function data4(n)
      return

endclass

class _NOTIFYICONDATA of ccsStructBase(88)
/*
   Windows API Shell_NotifyIconA() _NOTIFYICONDATA structure definition.
*/

   class::cbSize((this.sizeOf))                                               // set its size

// Method ======================================================================

   function cbSize(n)
      return iif(pcount() = 0,this.GetLong(0),this.SetLong(0,n))

// Method ======================================================================

   function hWnd(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function uId(n)
      return iif(pcount() = 0,this.GetLong(8),this.SetLong(8,n))

// Method ======================================================================

   function uFlags(n)
      return iif(pcount() = 0,this.GetLong(12),this.SetLong(12,n))

// Method ======================================================================

   function uCallBackMessage(n)
      return iif(pcount() = 0,this.GetLong(16),this.SetLong(16,n))

// Method ======================================================================

   function nIcon(n)
   /*
      Note:

         This member cannot be named hIcon. It generates an error that is
         unfathomable.

         This member MUST use the ccs_StructBase::SetHandle() method as an icon
         handle is an unsigned long value. The ccsStructBase::setLong() method
         only supports signed long values.

         i.e. bitZrshift()/Bitrshift() & unsigned/signed long values

   */
      return iif(pcount() = 0,this.GetLong(20),this.SetHandle(20,n))

// Method ======================================================================

   function szTip(c)
      return iif(pcount() = 0,this.AnsiString(24,64),;
                              this.AnsiString(24,64,c))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[hWnd:],ltrim(str(this.hWnd())) at 30
         ?[uId:],ltrim(str(this.uId())) at 30
         ?[uFlags:],ltrim(str(this.uFlags())) at 30
         ?[uCallBackMessage:],ltrim(str(this.uCallBackMessage())) at 30
         ?[hIcon:],ltrim(str(this.hIcon())) at 30
         ?[szTip:],ltrim(this.szTip()) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

class tagPoint of ccsStructBase(8)
/*
   Windows API GetCursorPos() tagPoint structure definition.
*/

// Method ======================================================================

   function x(n)
      return iif(pcount() = 0,this.GetLong(0),this.SetLong(0,n))

// Method ======================================================================

   function y(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[x:],ltrim(str(this.x())) at 30
         ?[y:],ltrim(str(this.y())) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

class _RECT of ccsStructBase(16)
/*
   Windows API GetCursorPos() _RECT structure definition.
*/

// Method ======================================================================

   function left(n)
      return iif(pcount() = 0,this.GetLong(0),this.SetLong(0,n))

// Method ======================================================================

   function top(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function right(n)
      return iif(pcount() = 0,this.GetLong(8),this.SetLong(8,n))

// Method ======================================================================

   function bottom(n)
      return iif(pcount() = 0,this.GetLong(12),this.SetLong(12,n))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[left:],ltrim(str(this.left())) at 30
         ?[top:],ltrim(str(this.top())) at 30
         ?[right:],ltrim(str(this.right())) at 30
         ?[bottom:],ltrim(str(this.bottom())) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

class VersionInfo of ccsStructBase(156)
/*
   Windows API GetVersionExA() lpVersionInfo structure definition.
*/

   class::dwOSVersionInfoSize((this.SizeOf))                                  // set its size

// Method ======================================================================

   function dwOSVersionInfoSize(n)
      return iif(pcount() = 0,this.GetLong(0),this.SetLong(0,n))

// Method ======================================================================

   function dwMajorVersion(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function dwMinorVersion(n)
      return iif(pcount() = 0,this.GetLong(8),this.SetLong(4,n))

// Method ======================================================================

   function dwBuildNumber(n)
      return iif(pcount() = 0,this.GetLong(12),this.SetLong(4,n))

// Method ======================================================================

   function dwPlatformId(n)
      return iif(pcount() = 0,this.GetLong(16),this.SetLong(4,n))

// Method ======================================================================

   function szCSDVersion(c)
      return iif(pcount() = 0,this.AnsiString(20,128),;
                              this.AnsiString(20,128,c))

// Method ======================================================================

   function wServicePackMajor(n)
      return iif(pcount() = 0,this.GetWord(148),this.SetWord(148,n))

// Method ======================================================================

   function wServicePackMinor(n)
      return iif(pcount() = 0,this.GetWord(150),this.SetWord(150,n))

// Method ======================================================================

   function wSuiteMask(n)
      return iif(pcount() = 0,this.GetWord(152),this.SetWord(152,n))

// Method ======================================================================

   function wProductType(n)
   //
   // The next 2 bytes are defined by TYPE_WORD. wProductType is in the low byte.
   //
   // With Vista SP1 the reserved high byte is used for the first time! This means
   // the test for the wProductType MUST be a bit test not an integer test.
   //

      return iif(pcount() = 0,this.GetWord(154),this.SetWord(154,n))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[dwMajorVersion:],ltrim(str(this.dwMajorVersion())) at 30
         ?[dwMinorVersion:],ltrim(str(this.dwMinorVersion())) at 30
         ?[dwBuildNumber:],ltrim(str(this.dwBuildNumber())) at 30
         ?[dwPlatformId:],ltrim(str(this.dwPlatformId())) at 30
         ?[szCSDVersion:],trim(this.szCSDVersion()) at 30
         ?[wServicePackMajor:],ltrim(str(this.wServicePackMajor())) at 30
         ?[wServicePackMinor:],ltrim(str(this.wServicePackMinor())) at 30
         ?[wSuiteMask:],ltrim(str(this.wSuiteMask())) at 30
         ?[wProductType:],ltrim(str(this.wProductType())) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

class SystemInfo of ccsStructBase(36)
/*
   Windows API GetSystemInfo() and GetNativeSystemInfo() lpSystemInfo structure definition.
*/

// Method ======================================================================

   function wProcessorArchitecture(n)
      return iif(pcount() = 0,this.GetWord(0),this.SetWord(0,n))

// Method ======================================================================

   function wReserved(n)
      return iif(pcount() = 0,this.GetWord(2),this.SetWord(2,n))

// Method ======================================================================

   function dwPageSize(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function lpMinimumApplicationAddress(n)
      return iif(pcount() = 0,this.GetLong(8),this.SetLong(8,n))

// Method ======================================================================

   function lpMaximumApplicationAddress(n)
      return iif(pcount() = 0,this.GetLong(12),this.SetLong(12,n))

// Method ======================================================================

   function dwActiveProcessorMask(n)
      return iif(pcount() = 0,this.GetLong(16),this.SetLong(16,n))

// Method ======================================================================

   function dwNumberOfProcessors(n)
      return iif(pcount() = 0,this.GetLong(20),this.SetLong(20,n))

// Method ======================================================================

   function dwProcessorType(n)
      return iif(pcount() = 0,this.GetLong(24),this.SetLong(24,n))

// Method ======================================================================

   function dwAllocationGranularity(n)
      return iif(pcount() = 0,this.GetLong(28),this.SetLong(28,n))

// Method ======================================================================

   function wProcessorLevel(n)
      return iif(pcount() = 0,this.GetWord(32),this.SetWord(32,n))

// Method ======================================================================

   function wProcessorRevision(n)
      return iif(pcount() = 0,this.GetWord(34),this.SetWord(34,n))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[wReserved:],ltrim(str(this.wReserved())) at 30
         ?[wProcessorArchitecture:],ltrim(str(this.wProcessorArchitecture())) at 30
         ?[dwPageSize:],ltrim(str(this.dwPageSize())) at 30
         ?[lpMinimumApplicationAddress:],ltrim(str(this.lpMinimumApplicationAddress())) at 30
         ?[lpMaximumApplicationAddress:],ltrim(str(this.lpMaximumApplicationAddress())) at 30
         ?[dwActiveProcessorMask:],ltrim(str(this.dwActiveProcessorMask())) at 30
         ?[dwNumberOfProcessors:],ltrim(str(this.dwNumberOfProcessors())) at 30
         ?[dwProcessorType:],ltrim(str(this.dwProcessorType())) at 30
         ?[dwAllocationGranularity:],ltrim(str(this.dwAllocationGranularity())) at 30
         ?[wProcessorLevel:],ltrim(str(this.wProcessorLevel())) at 30
         ?[wProcessorRevision:],ltrim(str(this.wProcessorRevision())) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

class _MEMORYSTATUSEX of ccsStructBase(64)
/*
   Windows API GlobalMemoryStatusEx() lpBuffer structure definition.
*/

   class::dwLength((this.SizeOf))                                             // set its size

// Method ======================================================================

   function dwLength(n)
      return iif(pcount() = 0,this.GetLong(0),this.SetLong(0,n))

// Method ======================================================================

   function dwMemoryLoad(n)
      return iif(pcount() = 0,this.GetLong(4),this.SetLong(4,n))

// Method ======================================================================

   function ullTotalPhys(n)
      return iif(pcount() = 0,this.GetUQuadword(8),this.SetDWORDLONG(8,n))

// Method ======================================================================

   function ullAvailPhys(n)
      return iif(pcount() = 0,this.GetUQuadword(16),this.SetDWORDLONG(16,n))

// Method ======================================================================

   function ullTotalPageFile(n)
      return iif(pcount() = 0,this.GetUQuadword(24),this.SetDWORDLONG(24,n))

// Method ======================================================================

   function ullAvailPageFile(n)
      return iif(pcount() = 0,this.GetUQuadword(32),this.SetDWORDLONG(32,n))

// Method ======================================================================

   function ullTotalVirtual(n)
      return iif(pcount() = 0,this.GetUQuadword(40),this.SetDWORDLONG(40,n))

// Method ======================================================================

   function ullAvailVirtual(n)
      return iif(pcount() = 0,this.GetUQuadword(48),this.SetDWORDLONG(48,n))

// Method ======================================================================

   function ullAvailExtendedVirtual(n)
      return iif(pcount() = 0,this.GetUQuadword(56),this.SetDWORDLONG(56,n))

// Method ======================================================================

   function showStructure()

      if ([runtime]$lower(version(0))) = 0
         ?
         ?"["+this.className+" structure"+[ (]+ltrim(str(this.SizeOf))+[ bytes)]+"]"
         ?
         ?[dwMemoryLoad:],ltrim(str(this.dwMemoryLoad())) at 30
         ?[ullTotalPhys:],ltrim(str(this.ullTotalPhys())) at 30
         ?[ullAvailPhys:],ltrim(str(this.ullAvailPhys())) at 30
         ?[ullTotalPageFile:],ltrim(str(this.ullTotalPageFile())) at 30
         ?[ullAvailPageFile:],ltrim(str(this.ullAvailPageFile())) at 30
         ?[ullTotalVirtual:],ltrim(str(this.ullTotalVirtual())) at 30
         ?[ullAvailVirtual:],ltrim(str(this.ullAvailVirtual())) at 30
         ?[ullAvailExtendedVirtual:],ltrim(str(this.ullAvailExtendedVirtual())) at 30
         ?
      endif

      return

// Method ======================================================================

   function release()
      super::release()
      return

endclass

Class ccsStructBase(nSize) Of String

/*

Marc Van den Berghen

Base class for all structures.

From API_Structs.cc

Andrew Shimmin

Renamed and modified.

*/

   // -------------------------------------

   // Constants

   #define GMEM_FIXED 0

   // -------------------------------------

   // Parameters

   if empty(nSize)
      nSize=128
   endif
   // Enlarge to WordBorder for Unicode
   nSize = iif((nSize % 2) == 1, nSize + 1, nSize)

   // -------------------------------------

   // Custom Properties

   this.String = Replicate(Chr(0),nSize / 2) // Initialize with 0
   this.SizeOf = nSize                       // size of struct

   // -------------------------------------

   // API Externs

   if type("GlobalAlloc") # "FP"
      extern chandle GlobalAlloc(;
         cuint, ;
         clong ;
         );
         kernel32
   endif

   if type("GlobalFree") # "FP"
      extern chandle GlobalFree(;
         chandle ;
         ) ;
         kernel32
   endif

   if type("GetCurrentProcess") # "FP"
      extern chandle GetCurrentProcess(;
         cvoid ;
         ) ;
         kernel32
   endif

   if type("WriteProcessMemory") # "FP"
      extern clogical WriteProcessMemory(;
         chandle, ;
         cptr, ;
         cstring, ;
         clong, ;
         cptr ;
         ) ;
         kernel32
   endif

   if type("WriteProcessMemoryP") # "FP"
      extern clogical WriteProcessMemoryP(;
         chandle, ;
         cptr, ;
         cptr, ;
         clong, ;
         cptr ) ;
         kernel32 ;
      from 'WriteProcessMemory'
   endif

// Method ======================================================================

   function GetLong(nIndex)

      // Get the long(dword) value

      local n
      n = this.getByte(nIndex) + bitlshift(this.getByte(nIndex + 1), 8) ;
                               + bitlshift(this.getByte(nIndex + 2), 16) ;
                               + bitlshift(this.getByte(nIndex + 3), 24)

   return n

// Method ======================================================================

   function SetLong(nIndex, nValue)

      // Set the long value at the desired position in the structure

      this.setByte(nIndex, bitand(nValue, 0xFF))
      this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
      this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
      this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))

   return

// Method ======================================================================

   function SetULong(nIndex, nValue)

      // Set the unsigned long value at the desired position in the structure

      this.setByte(nIndex, bitand(nValue, 0xFF))
      this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
      this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
      this.setByte(nIndex + 3, bitzrshift(bitand(nValue, 0xFF000000), 24))

   return

// Method ======================================================================

   function GetHandle(nIndex)

      // Get the windows handle value

   return (class::GetLong(nIndex))

// Method ======================================================================

   function SetHandle(nIndex, nValue)

      // Set the windows handle at the desired position in the structure

      class::SetULong(nIndex, nValue)

   return

// Method ======================================================================

   function GetLogical(nIndex)

      local n,lValue

      // get the word value

      n = this.GetByte(nIndex) ;
        + bitlshift(this.GetByte(nIndex + 1), 8)

      lValue = (n = true)

   return lValue

// Method ======================================================================

   function SetLogical(nIndex,lValue)

      local nValue

      nValue = iif(lValue,1,0)

      // set the word value

      this.SetByte(nIndex, bitand(nValue, 0xFF))                              // position in the structure
      this.SetByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))

   return

// Method ======================================================================

   function GetDWORDLONG(nIndex)

      // Get the DWORDLONG value

      local n
      n = this.getByte(nIndex) ;
        + bitlshift(this.getByte(nIndex + 1),  8) ;
        + bitlshift(this.getByte(nIndex + 2), 16) ;
        + bitlshift(this.getByte(nIndex + 3), 24) ;
        + bitlshift(this.getByte(nIndex + 4), 32) ;
        + bitlshift(this.getByte(nIndex + 5), 40) ;
        + bitlshift(this.getByte(nIndex + 6), 48) ;
        + bitlshift(this.getByte(nIndex + 7), 56)

   return n

// Method ======================================================================

   function GetUQuadword(nIndex)

      // Get the unsigned Quadword value

      local n,bValue,b

      n = 0

      for i = 0 to 7

         bValue = this.getByte(nIndex+i)

         for b = 0 to 7
            n += iif(bitset(bValue,b),2**(i*8+b),0)
         endfor

      endfor

   return n

// Method ======================================================================

   function SetDWORDLONG(nIndex,nValue)

      // Set the DWORDLONG value at the desired position in the structure

      this.setByte(nIndex, bitand(nValue, 0xFF))
      this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
      this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
      this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))

      this.setByte(nIndex + 4, bitrshift(bitand(nValue, 0xFF00000000), 32))
      this.setByte(nIndex + 5, bitrshift(bitand(nValue, 0xFF0000000000), 40))
      this.setByte(nIndex + 6, bitrshift(bitand(nValue, 0xFF000000000000), 48))
      this.setByte(nIndex + 7, bitrshift(bitand(nValue, 0xFF00000000000000), 56))

   return

// Method ======================================================================

   function Release()

      // cleanup

      this.String = ''
      release object this

   return

// Method ======================================================================

   function Member(nIndex, nValue)

      /*

      The following method is used to overload the member-functions
      of the subclassed structures, so that the same name can be used
      to set or to get the values (by checking the number of parameters)

      */

      if pcount() == 1
         return this.getlong(nIndex)
      else
         this.setlong(nIndex, nValue)
      endif

   return nValue

// Method ======================================================================

   function SetText

      /*

      Windows often uses pointers to strings in structures. The
      following method was specially designed to enable the
      dbase developper to use such structures. It allocates memory
      to hold the string and gets the address of this memory
      block from the OS.

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function to ensure correct behaviour with
         diacritical characters.

      */

      // Pass pointer as reference so it can be changed

      parameters c, n, h

      local apierror, cResult, hprocess

      cResult = ""

      if type("c")="U"

         return cResult

      else

         if h # 0

            // release the old memory-block so that
            // it can be reused later (and to
            // avoid memory-leaks)

            cResult = GlobalFree(h)

         endif

         if type("c") == "C"

            h = globalalloc(GMEM_FIXED, len(c) + 1) // Get the address
            if h=0
               msgbox("Memory allocation failed", " Fatal Error", 0)
            else
               hprocess = GetCurrentProcess()
               // Write the string to the reserved memory-location
               apierror = WriteProcessMemory(hprocess, h, oem(c + chr(0)), len(c) + 1, 0)
               if not apierror == true
                  msgbox("Pointer creation failed", " Fatal Error", 0)
               else
                  cResult = str(h)
                  this.setlong(n, h)
               endif
            endif
         else
            this.setlong(n, c)
         endif
      endif

      // Return either empty string (on error) or pointer to the string

   return cResult

// Method ======================================================================

   function GetText(n, h)

      /*

      This method is not fully implemented yet !!!!!!!
      (the value of the string at the position pointed to is not
      yet returned)

      */

      local cResult
      cResult = ""
      h = this.getlong(n)

   return

// Method ======================================================================

   function AnsiString(nIndex, nMax, cValue)

      /*

      The following function is used to get and set string-values
      in a structure

      !!! Attention : it handles only ANSI-Characters (singlebyte)
      Unicode not yet implemented

      nMax : must contain the maximum space accorded to the specific
            string-member (used to avoid overwriting of the other members)

      */

      if pcount() == 2
         return this.GetAnsiString(nIndex, nMax)
      else
         this.SetAnsiString(nIndex, nMax, cValue)
      endif

   return cValue

// Method ======================================================================

   function SetAnsiString(nIndex, nMax, cValue)

      /*

      2007/05/11 Marc Van den Berghen

         Changed to use oem() function to ensure correct behaviour with diacritical characters.

      2011/03/11 Andrew Shimmin

         Changed to reset the whole string before before setting the string value.

         This is required when a structure string member is reused with a smaller
         string than its predecessor.

         The TaskBarTray.cc can do the above with the szTip member of the _NotifyIconData structure.

      */

      local i,cReset

      cReset = replicate(oem(chr(0)),nMax)

      for i = 0 to nMax-1
         this.SetByte(nIndex+i,asc(cReset.charAt(i)))                         // reset string
      endfor

      cvalue = oem(cvalue)
      nMax = iif(len(cvalue) < nMax-1, len(cValue) - 1, nMax - 2)
      for i = 0 to nMax
         this.SetByte(nIndex + i, asc(cValue.charAt(i)))
      endfor

   return

// Method ======================================================================

   function GetAnsiString(nIndex, nMax)

      // Assuming null-termination

      local cValue, zw

      cvalue = ''

      for i = 0 to nMax-1
         zw = this.GetByte(nIndex + i)
         if zw # 0
            cValue += chr(zw)
         else
            i = nMax
         endif
      endfor

   return ansi(cValue)

// Method ======================================================================

   function WordMember(nIndex, nValue)

      // Get and set two-byte values (words)

      if pCount() == 1
         return this.GetWord(nIndex)
      else
         this.SetWord(nIndex, nValue)
      endif

   return nValue

// Method ======================================================================

   function GetWord(nIndex)

      // get the dword value

      local n

      n = this.GetByte(nIndex) ;
        + bitlshift(this.GetByte(nIndex + 1), 8)

   return n

// Method ======================================================================

   function setWord(nIndex, nValue)

      // set the word value at the desired

      this.SetByte(nIndex, bitand(nValue, 0xFF)) // position in the structure
      this.SetByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))

   return

// Method ======================================================================

   function FillStruct(lp_str)

      local hProcess, apiError

      hprocess = GetCurrentProcess()

      apiError = WriteProcessMemoryP(hProcess, this.String, lp_str, this.SizeOf, 0)
      if not apiError == true
         msgbox("Pointer not valid", " Fatal Error", 0)
      endif

   return

endclass
