/*
-----------------------------------------------------------------------
Program........: TableCopy.cc
Programmer.....: Christer Persson
Date...........: 2001-09-15
Version........: 1.0
Written for....: dB2k
Rev. History...: 2001-09-15 - Original version 1.0

Usage..........: Transfers properties and RIrules from source table to 
					  destination
Example........:

set procedure to TableCopy.cc additive
oTC=new TableCopy()
oTC.source='source1.dbf'
oTC.destination='dest1.dbf'
oTC.copyStructure()
-----------------------------------------------------------------------
*/
class TableCopy

PROTECT CheckFieldnames, WriteProp, Setup

	this.destination=''			// Destination table
   this.fieldNames=new array()		// List of names of fields to copy
   this.production=true			// If true, production index will be copied
   this.properties=true			// If true, standard and custom properties will be transfered
   this.RIrules=false			// If true, RI rules will be transfered
   this.source=''					// Source table
   this.tempPath=''				// Path to store temporary files

   function Setup
      if this.source==''
      	MsgBox('No source table', 'Error', 16)
         return false
      endif
      if this.destination==''
      	MsgBox('No destination table', 'Error', 16)
         return false
      endif
   	return true

   function copyStructure
   	local oDesign, cTempTable, oTDsource, oSource
      if not class::Setup()
      	return
      endif
      // Source table
      oSource=new query()
      oSource.sql='select * from "'+this.source+'"'
      oSource.active=true
      // Check list of fields
      class::CheckFieldnames(oSource)
      // Create structure table
      cTempTable=FUNIQUE(this.tempPath+'????????.dbf')
      cTempFile='"'+cTempTable+'"'
      CREATE TABLE &cTempFile. (;
      FIELD_NAME CHAR(32), ;
      FIELD_TYPE CHAR(32), ;
      FIELD_LEN NUMERIC(6,0), ;
      FIELD_DEC NUMERIC(6,0), ;
      FIELD_IDX CHAR(1))
		oDesign=new query()
      oDesign.sql='select * from "'+cTempTable+'"'
      oDesign.active=true
      // Add fields
      for i=1 to this.fieldNames.size
      	if UPPER(this.fieldNames[i])='_DBASELOCK'
         	loop
         endif
         oDesign.rowset.beginAppend()
         oDesign.rowset.fields['FIELD_NAME'].value=this.fieldNames[i]
         oDesign.rowset.fields['FIELD_TYPE'].value=;
         oSource.rowset.fields[this.fieldNames[i]].type
         oDesign.rowset.fields['FIELD_LEN'].value=;
         oSource.rowset.fields[this.fieldNames[i]].length
         oDesign.rowset.fields['FIELD_DEC'].value=;
         oSource.rowset.fields[this.fieldNames[i]].decimalLength
         oDesign.rowset.save()
      endfor
      oDesign.active=false
      oSource.active=false
      // Create destination table
      // Create ... from ... removes any duplicate fieldnames
      // so no special code is required for that
      create (this.destination) from (cTempTable)
      use
      // Delete structure table
      try
         delete table (cTempTable)
      catch (Exception e)
      endtry
      // If production index should be transfered
      if this.production
         oTDsource=new TableDef()
         oTDsource.tableName=this.source
         oTDsource.load()
         for i=1 to oTDsource.indexes.size
         	try
      		_app.databases[1].createIndex(this.destination, oTDsource.indexes[i])
            catch (Exception e)
            	// Error if required fields are not present.
            endtry
         endfor
      endif
      // If properties should be transfered
      if this.properties
	      class::WriteProp(this.destination, this.source, this.fieldNames, this.RIrules)
      endif
   	return

   function copy
   	class::copyStructure()
      use (this.destination) excl
      append from (this.source) reindex
      use
   	return

   function transferProperties
   	local oSource
      if not class::Setup()
      	return
      endif
      // Source table
      oSource=new query()
      oSource.sql='select * from "'+this.source+'"'
      oSource.active=true
      // Check list of fields
      class::CheckFieldnames(oSource)
      oSource.active=false
      class::WriteProp(this.destination, this.source, this.fieldNames, this.RIrules)
   	return

   function CheckFieldnames(oSource)
   	local i, j, bExist
      if this.fieldNames.size=0
      	// If no fields specified, select all fields
      	for i=1 to oSource.rowset.fields.size
         	this.fieldNames.add(oSource.rowset.fields[i].fieldName)
         endfor
      else
      	// Remove fieldnames not present in source
      	for i=this.fieldNames.size to 1 step -1
	        	bExist=false
         	for j=1 to oSource.rowset.fields.size
            	if UPPER(oSource.rowset.fields[j].fieldname)=;
               UPPER(this.fieldNames[i])
               	bExist=true
                  exit
               endif
            endfor
            if not bExist
               this.fieldNames.delete(i)
               this.fieldNames.size--
            endif
         endfor
      endif
   	return

   function WriteProp(cDestination, cSource, oFields, bRI)
   	// cDestination is the name of the destination table
      // cSource is the name of the source table
      // oFields is the array of fieldnames to include
      // If bRI is true, RI rules will be transfered
      local oTPsource, oTPdest, i, j, nSource, nDest, oProperty
      set procedure to Tableprop.cc additive
      oTPsource=new TableProp()
      oTPsource.filename=cSource
      oTPsource.readHeader()

      oTPdest=new TableProp()
      oTPdest.filename=cDestination
      oTPdest.readHeader()

      for i=1 to oTPsource.StandardProps.size
         // nSource is the number of the field in the source
         nSource=oTPsource.StandardProps[i].fieldnumber
         // Is the field included in list?
         if oFields.scan(oTPsource.TrimNull(oTPsource.fields[nSource].fieldName))=0
         	loop
         endif
         // nDest is the number of the field in the destination matching 
         // the fieldname from the source
         nDest=0
         for j=1 to oTPdest.fields.size
            if oTPdest.fields[j].fieldname=;
            oTPsource.fields[nSource].fieldname
               nDest=j
               exit
            endif
         endfor

         if nDest<>0
            // Transfer property only if the same type and length
            if oTPsource.fields[nSource].fieldtype=;
            oTPdest.fields[nDest].fieldtype and ;
            oTPsource.fields[nSource].length=;
            oTPdest.fields[nDest].length and ;
            oTPsource.fields[nSource].decimals=;
            oTPdest.fields[nDest].decimals
               oProperty=new StandardProp()
               oProperty.fieldnumber=nDest
               oProperty.propertyNumber=;
               oTPsource.StandardProps[i].propertyNumber
               oProperty.value=oTPsource.StandardProps[i].value
               oTPdest.Standardprops.add(oProperty)
            endif
         endif
      endfor

      for i=1 to oTPsource.CustomProps.size
         nSource=oTPsource.CustomProps[i].fieldnumber
         // Is the field included in list?
         if oFields.scan(oTPsource.TrimNull(oTPsource.fields[nSource].fieldName))=0
         	loop
         endif
         // nDest is the number of the field in the destination matching 
         // the fieldname from the source
         nDest=0
         for j=1 to oTPdest.fields.size
            if oTPdest.fields[j].fieldname=;
            oTPsource.fields[nSource].fieldname
               nDest=j
               exit
            endif
         endfor
         if nDest<>0
            // Transfer property only if the same type and length
            if oTPsource.fields[nSource].fieldtype=;
            oTPdest.fields[nDest].fieldtype and ;
            oTPsource.fields[nSource].length=;
            oTPdest.fields[nDest].length and ;
            oTPsource.fields[nSource].decimals=;
            oTPdest.fields[nDest].decimals
               oProperty=new CustomProp()
               oProperty.fieldnumber=nDest
               oProperty.name=oTPsource.CustomProps[i].name
               oProperty.value=oTPsource.CustomProps[i].value
               oTPdest.Customprops.add(oProperty)
            endif
         endif
      endfor

      if bRI
      	// If RI rules should be transfered
         oTPdest.RefIntegProps=oTPsource.RefIntegProps
      endif

      oTPdest.writeHeader()
      return


endclass
