/*
   Class:   FontGetter
   Author:  Daniel Howard sproket@total.net

   History: 
      1.0   December 12, 1998
            Initial Release

   Properties:
      Font     -  This is an object representation of the font string.
                  eg "Arial,10,B" would be
                  font.FontName = "Arial"
                  font.FontSize = 10
                  font.FontBold = true
                  font.FontItalic = false
                  font.FontStrikeout = false
                  font.FontUnderline = false
   Notes:   

      This File contains a Font Class which handles the font 
      information. You may find it useful in other contexts.

      See TreeView.WFM for an example of it's use.

*----------------------------------------------------------------------
This software is donated to the public domain. You may modify 
this code in anyway you wish however the author reserves the 
right to whatever official version is published. If you have 
further ideas/bug reports/additions then contact the author and 
your name will be added to the contributors if your idea is 
implemented.

The author accepts no responsibility for the use or consequences 
of the use of this software.

*/

class FontGetter(parentObj, name) of DCONTAINER(parentObj, name) custom
   with (this)
      left = 1.7143
      top = 2.0909
      width = 31.6667
      height = 3.5
      metric = 0	// Chars
   endwith


   this.RECTANGLE1 = new FG_RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 0.5
      top = 0
      width = 29.6667
      height = 3.05
      fontSize = 8
      text = "Font"
   endwith


   this.FontField = new FG_ENTRYFIELD(this)
   with (this.FontField)
      height = 1
      left = 1.2857
      top = 0.75
      width = 28.1429
      OnChange = Class::FontField_OnChange
   endwith


   this.FONTBUTTON = new FG_WRENCHBUTTON(this)
   with (this.FONTBUTTON)
      onClick = class::FONTBUTTON_ONCLICK
      height = 0.8182
      left = 25.7143
      top = 0.85
      width = 3.4286
   endwith

   this.SAMPLETEXT = new dTEXT(this)
   with (this.SAMPLETEXT)
      height = 1
      left = 1.1429
      top = 1.8
      width = 28
      fontSize = 8
      text = "Sample"
   endwith


   this.Font = New Font()
 
   this.Setter.OnChange := Class::Setter_OnChange
   this.Setter.CanChange := Class::Setter_CanChange
   this.Setter.BeforeGetValue := Class::Setter_BeforGetValue

   this.Setter.Value := "Arial,10"
   this.FontField.OnChange := Class::FontField_OnChange


   function FONTBUTTON_onClick
   local cFont
         
      cFont = GetFont()

      if not empty(cFont)
         this.Parent.Setter.Value := cFont
      endif
   return

   function FontField_OnChange
      this.Parent.Setter.Value := this.Value
      this.Value := this.Parent.Setter.Value
   return
               
   Function SetFont
      this.FontField.Value := this.Font.AsString()
      this.SampleText.FontName = this.Font.FontName
      this.SampleText.FontSize = this.Font.FontSize
      this.SampleText.FontBold = this.Font.FontBold
      this.SampleText.FontItalic = this.Font.FontItalic
      this.SampleText.FontStrikeout = this.Font.FontStrikeout
      this.SampleText.FontUnderline = this.Font.FontUnderline
   return 

   Function Setter_OnChange
      local cFont
      cFont = this.Value
      this.Owner.Font.DefineFrom( (cFont) )
      this.Owner.SetFont()
   return

   // Make sure it's a valid font.
   Function Setter_CanChange
   para xValue

      local oFont
      oFont = New Font()
      oFont.DefineFrom( (xValue) )

      local f,t
      f = New Form()
      t = New text(f)

      local lValid
      lValid = true
      try
         t.FontName := oFont.FontName
         t.FontSize := oFont.FontSize
         t.FontBold := oFont.FontBold
         t.FontItalic := oFont.FontItalic
         t.FontStrikeout := oFont.FontStrikeout
         t.FontUnderline := oFont.FontUnderline
      catch(Exception e)
         lValid := false
      endtry

      if not lValid
         msgbox("Invalid font assignment!","Font Getter")
      endif
   return ( lValid )

   Function Setter_BeforGetValue
      // cFont returns family which I don't use so I remove it here.
      local cFont
      cFont = this.Value

      local nCommas
      nCommas = FG_Extras::CharCount(cFont,",")

      if nCommas == 3   
         * Arial,10,BI,Swiss
         * Arial,10,BIUS,Swiss
         local n
         n = rat(",",cFont)
         cFont := left(cFont,n-1)
      elseif nCommas == 2   
         // This is a little ambiguous since I could have a setting
         // for Bold, Italics, etc or a family name. How do I tell
         // the differece? For now I'll simply say that if the length
         // is <= 4 then it's NOT a family name.
         * Arial,10,Swiss ?
         * Arial,10,B     ?
         local cStr
         cStr = upper(FG_Extras::StrItem(3,cFont))
         if len(cStr) > 4
            local n
            n = rat(",",cFont)
            cFont := left(cFont,n-1)
         endif
      endif
   return (cFont)
endclass


Class FG_Rectangle(parentObj) of dRectangle(parentObj)
EndClass

Class FG_EntryField(parentObj) of dEntryField(parentObj)
protect datalink,onChange
EndClass

class FG_WrenchButton(parentObj) of dPushButton(parentObj) custom
protect speedbar,text,onClick
   with (this)
      speedBar = true
      upBitmap = "RESOURCE #144"
      text = ""
   endwith
endclass

// This class hold/handles font information
Class Font

   this.FontName = ""
   this.FontSize = 0
   this.FontBold = false
   this.FontItalic = false
   this.FontStrikeout = false
   this.FontUnderline = false
   this.FontFamily = "" // Not used 


   // Takes a font string or an object and defines this object's
   // properties from it.
   Function DefineFrom
   parameter xFont

      if type("xFont") == "O"        
         this.FontName := xFont.FontName
         this.FontSize := xFont.FontSize
         this.FontItalic := xFont.FontItalic
         this.FontStrikeout := xFont.FontStrikeout
         this.FontUnderline := xFont.FontUnderline
         this.FontBold := xFont.FontBold
      else        // String
         this.FontName = FG_Extras::StrItem(1,xFont)
         this.FontSize = Val(FG_Extras::StrItem(2,xFont))
         if FG_Extras::CharCount(xFont,",") >= 2
            this.FontBold = "B" $ upper(FG_Extras::StrItem(3,xFont))
            this.FontItalic = "I" $ upper(FG_Extras::StrItem(3,xFont))
            this.FontStrikeout = "S" $ upper(FG_Extras::StrItem(3,xFont))
            this.FontUnderline = "U" $ upper(FG_Extras::StrItem(3,xFont))
         else
            this.FontBold = false
            this.FontItalic = false
            this.FontStrikeout = false
            this.FontUnderline = false
         endif
      endif              
   return


   // This returns the current object's properties as a string
   // in the same format that the GetFont() would return.
   Function AsString
   local cStyle
   local cFont

      cStyle = ""
      cStyle += iif(this.FontBold,"B","")
      cStyle += iif(this.FontItalic,"I","")
      cStyle += iif(this.FontUnderline,"U","")
      cStyle += iif(this.FontStrikeout,"S","")

      if empty(cStyle)
         cFont = this.FontName+","+str(this.FontSize,2,0)
      else
         cFont = this.FontName+","+str(this.FontSize,2,0)+","+cStyle
      endif
   return ( cFont )

   // This set's a passed object's font information from this object's
   // font information.
   Function SetObjectFont(o)
      o.FontName = this.FontName
      o.FontSize = this.FontSize
      o.FontBold = this.FontBold
      o.FontItalic = this.FontItalic
      o.FontStrikeout = this.FontStrikeout
      o.FontUnderline = this.FontUnderline
   return

endclass

Class FG_Extras
   function CharCount(cStr,cChar)
   local j
   local count
   local nLen
   
      if Class::ValType(cChar) <> 'C' .or. Class::Valtype(cStr) <> 'C'
         msgbox("Invalid parameters :"+program()+'.')
         return ( 0 )
      endif
   
      nLen  = len(cChar)
      count = 0
      for j = 1 to len(cStr)
         if cChar = substr(cStr,j,nLen)
            count = count + 1
         endif
      next j
   return ( count )   

   function StrItem(nItem,cStr)
   local cRet
      cRet = Class::StrExtract(nItem,cStr,",")
   return ( cRet )

   // This version is a little buggy but it will work perfecly 
   // for this class.
   // the bug is if you pass a string like "one,,three"
   // StrExtract(2,"one,,three",",") doesn't return an empty
   // string. I had a good version of this somewhere.....
   function StrExtract(nItem,cStr,cToken)
   local p1,p2,j
   local cItem
   local cSubStr
      if Class::valtype(cStr) <> "C"
         return ""
      endif
   
      p1 = 1
      p2 = 0
      cSubStr = cStr
      cItem  = ""
      j = 1
      do while .not. empty(cSubStr)
         pos = at(cToken,cSubStr)-1
         if pos <= 0
            pos = len(cSubStr)
         endif
         cItem = left(cSubstr,pos)
         if j = nItem
            exit
         endif
         if pos < len(cSubStr) - 1
            cSubStr = substr(cSubStr,pos+2,len(cSubStr))
            if cToken == " "
               cSubStr = ltrim(cSubStr)
            endif
         else
            cSubStr = ""
            cItem   = ""
         endif
         j = j + 1
      enddo
   return (cItem)

   Function Valtype
   para xValue
   return type("xValue")
EndClass
