class kmQuery of query
/*
	Custom query class that generates a warning message if
   an attempt is made to set a non-existant index.

   Do NOT use the IndexName property to set an index order.
   Instead, use the query's SetIndex() method.

   Written by Gary White
   May 4, 1999

   Usage:

   q = new Query()
   q.sql    := "select * from sometable"
   q.active := true
   q.SetIndex( "myindextag" )
     // if index tag doesn't exist, an error message
     // will occur.

*/
   protect indexName

   function execute
      local cString, bSuccess
      bSuccess = super::execute()
      if bSuccess
         cString = this.sql
         do
            cString := substr( cString, at( ' ', cString ) + 1 )
         until upper(left( cString, 4 )) == 'FROM'
         cString := ltrim( substr( cString, 5 ) )
         if ' ' $ cString
            cString := left( cString, at( ' ', cString ) -1 )
         endif
         this.t = new tabledef()
         this.t.tablename= cString
         this.t.database = q.database
         this.t.load()
         this.indicies = new array()
         for i = 1 to this.t.indexes.size
            this.indicies.add( upper( this.t.indexes[i].indexName ) )
         endfor
      endif
   return bSuccess

   function setIndex( cIndexName )
      private cExact
      cExact = set('exact')
      set exact on
      if this.indicies.scan( upper( cIndexName ) ) > 0
         this.indexName := cIndexName
      else
         msgbox('Index tag '+ cIndexName + ' not found', 'Error', 16 )
      endif
      set exact &cExact
   return upper( this.indexName ) == upper( cIndexName )

endclass

