/*
   Description:
 
      A custom control written by Romain Strieff. Through the use
      of API calls, you can set or get the position of the 3 keys
      CAPSLOCK, SCROLLLOCK and NUMLOCK.

 
   Programmers: 

      Romain Strieff 
 
   History: 
 
      03/01/2002 -- First appearance of this on the dBASE
                    newsgroups (posted by Romain)
      03/01/2002 -- Class made out of the original

   Instantiation: 
 
      set procedure to :dUFLP:LockKeys.cc additive
      k=new LockKeys()
      if k.GetScrollLock()
         k.SetScrollLock(false)
      endif
      if k.GetCapslock()
         k.SetCapsLock(false)
      endif

 
   Methods:

      GetCapsLock()
         Returns the actual position of the Capslock key
         this is called with no parameter

      GetNumLock()
         Returns the actual position of the Numlock key
         this is called with no parameter

      GetScrollLock()
         Returns the actual position of the Scrolllock key
         this is called with no parameter

      SetCapsLock(lOnOff)
         Sets the Capslock key to on or off
         Parameter logical TRUE for ON and FALSE for OFF

      SetNumLock(lOnOff)
         Sets the Numlock key to on or off
         Parameter logical TRUE for ON and FALSE for OFF

      SetScrollLock(lOnOff)
         Sets the Scrolllock key to on or off
         Parameter logical TRUE for ON and FALSE for OFF

      Release
         Releases the class from memory

    Protected methods:
        These cannot be called from outside, the methods
        above call these.

        Set()
           Does the actual setting for all keys
        Get() 
           Does the actual getting for all keys
*/

class LockKeys
   protect get, set
   //some constants that we need
   #ifndef KEYEVENTF_EXTENDEDKEY
     #define KEYEVENTF_EXTENDEDKEY 0x0001
   #endif
   #ifndef KEYEVENTF_KEYUP
      #define KEYEVENTF_KEYUP      0x0002
   #endif
   #ifndef VK_CAPITAL
      #define VK_CAPITAL           0x14
   #endif
   #ifndef VK_NUMLOCK
      #define VK_NUMLOCK           0x90
   #endif
   #ifndef VK_SCROLL
      #define VK_SCROLL            0x91
   #endif

   //and a couple of API functions
   if type("lGetKeyState") # "FP"
      extern cLogical lGetKeyState(cint) user32;
             from "GetKeyState"
   endif
   if type("keybd_event") # "FP"
      extern cVOID keybd_event;
             (cint, cint, cWORD, cWORD) user32
   endif


   // and here the methods, that's it 
   function SetCapslock(lOnOff)
      this.Set(VK_CAPITAL,lOnOff)
   return true  

   function SetScrolllock(lOnOff)
      this.Set(VK_SCROLL,lOnOff)
   return true
 

   function SetNumlock(lOnOff)
      this.Set(VK_NUMLOCK,lOnOff)
   return true

   function GetCapslock
   return this.Get(VK_CAPITAL)

   function GetScrolllock
   return this.Get(VK_SCROLL)

   function GetNumlock
   return this.Get(VK_NUMLOCK)

   function release
      release object this
   return

   //--------Set and Get functions----
   Function Get(nVirtualKey)
      sleep 0.15 //to make it 'stick'
   return lGetKeyState(nVirtualKey)

   Function Set
      parameters nVirtualKey,lOnOff
      //we switch only if it's in a different state

      if lGetKeyState(nVirtualKey) # lOnOff
         //Simulate Key Press
         keybd_event(nVirtualKey,;
               HTOI("45"),KEYEVENTF_EXTENDEDKEY,0)
         //Simulate Key Release
         keybd_event(nVirtualKey,;
               HTOI("45"),KEYEVENTF_EXTENDEDKEY+KEYEVENTF_KEYUP,0)
      endif
   return lGetKeyState(nVirtualKey)

endclass
/*
    End of File: LockKeys.cc
*/
