// Metric: 6 - Pixels only
/*
   pKENCAL3.CC
   Author  : Ken Mayer
   Original: August, 2000
   Updated : August, 2000
             March, 2004
             July, 2004 -- Pixels Only version

   Enhanced: Added buttons for +10 and -10 years, +3 and -3 months
             Dave Petonic, Redstone SoftBase Company
             * March 2004 update, added some minor changes
               to deal with some international issues.

   This is a (mostly) self-contained calendar class -- no
   ActiveX controls. It uses pushbuttons for dates and so on. 
   Note the dependencies section below -- for some of this
   to function properly, some other methods of the 
   dUFLP are involved.

   This is a copy of KenCal2.cc, but with a slightly
   different name for the container (pKenCalendar),
   and the whole thing is a bit larger, to accomodate
   larger buttons (larger text on the buttons). That's
   the ONLY real difference here between the two.

   Everything here is done with the OOP abilities of
   Visual dBASE and dB2K. 

   NOTE: This does not resize well ...

   Usage:
      set procedure to :dUFLP:kencal3.cc additive

      // in the forms designer, drop the container object
      // onto a form.

      The date can be SET in the form by referring to:

         form.pKenCalendar1.currentDate

         (If you do not do this, it will use the 
         current date)

      To GET the date you just refer to the same
      value: 
         form.pKenCalendar1.currentDate

      Assuming a form with a date entryfield:

      set procedure to myForm.wfm addi
      fMyForm = new MyFormForm()
      // use whatever value you need to -- you 
      // could use this in a modal form, for example,
      // and set the value to that in an entryfield,
      // and when the user closes the modal form,
      // reset the returned value to the entryfield ...
      fMyForm.pKenCalendar1.currentDate := {01/01/1999}
      // *** OPTIONAL LANGUAGE STUFF:
      fMyForm.pKenCalendarContainer1.language := "German (Standard)"
      fMyForm.pKenCalendarContainer1.TodayPushbutton.text := "Heute"
      // *** End of Optional Language Stuff
      fMyForm.readModal()
      // when done, before releasing the modal form:
      form.entryfield1.value = ;
               fMyForm.pKenCalendar1.currentDate
      // (where 'form.entryfield1' refers to the CALLING
      // form, and the entryfield that contains the date ...)

   NOTE: This stores a property to the container called beginDate
   which can be used if you include a "cancel" button ... -- you
   can reset the calendar control's currentDate property:

      form.pKenCalendar1.currentDate := ;
         form.pKenCalendar1.beginDate

   Example can be found in testcal3.wfm and a form that
   uses the control directly is Cal3Form.wfm

   COLOR PROPERTIES:
      The following custom color properties can be used
      to change the appearance of the calendar. These should
      be set probably in the form's open or onOpen event.

      form.pKenCalendar1.NORMALCOLOR = "somecolor"
          This can be used to set the color of the non-selected
          days. If you leave off a background color, it should
          use the default "BTNFACE" (grey).
      form.pKenCalendar1.SELECTEDCOLOR = "somecolor"
          This is used to set the selected day (current
          date) to a specific color so that it stands out.
          The default is red with a white background 
          ("red/white")
      form.pKenCalendar1.SUNDAYCOLOR = "somecolor"
          As above, this is used to display the first column
          in a specific color. The default is red on
          the standard button background ("RED/BTNFACE")
          NOTE: In this version, holidays that
          are defined in the "LoadHolidays" method of the
          container will appear in the "SundayColor"
          unless they are also the "selected" day ...
      form.pKenCalendar1.HOLIDAYCOLOR = "somecolor"
          As above, used specifically for Holidays, will
          override a sundaycolor setting ...

   LANGUAGE PROPERTIES:
      fMyForm.pKenCalendarContainer1.language := "German (Standard)"
          Note: this sets the language used based on 
                values found in the associative array in the
                beginning of DateEx.cc. The default is
                "English (United States)".
      fMyForm.pKenCalendarContainer1.TodayPushbutton.text := "Heute"
          This is important, as there is no good way for 
          me to find the correct text for this pushbutton.
          You could go further and change the text of
          the speedTips for the NextMonth, PreviousMonth,
          NextYear, PreviousYear, MonthPlus3, MonthMinus3,
          YearPlus10, YearMinus10 pushbuttons as well.


   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           dateex.cc
   **           holiday.cc
   **           time.cc
   **           hebrew.cc
   **           duflp.h <== not referred to by source alias

    DEPENDENCIES (Important for deploying an application
                  that uses this control):
       DATEEX.CC
       HOLIDAY.CC
       TIME.CC
       HEBREW.CC
       DUFLP.H
*/

/*
   -------------------------------------------------------
   The container class here is what does all the work ...
   There are some custom button/text control definitions 
   after the container definition.
   -------------------------------------------------------
*/
class pKenCalendar(parentObj) of CONTAINER(parentObj) custom
   set procedure to :dUFLP:dateex.cc additive
   set procedure to :dUFLP:holiday.cc additive

   with (this)
      width        := 175
      height       := 236
      metric       := 6	// Pixels
      transparent  := true
      onOpen       := class::CONTAINEROPEN
      onDesignOpen := class::CheckMetric
   endwith

   #define TITLEROW  51
   #define ROW1      73
   #define ROW2      95
   #define ROW3     117
   #define ROW4     138
   #define ROW5     161
   #define ROW6     183
   #define COL1       7
   #define COL2      29
   #define COL3      51
   #define COL4      73
   #define COL5      95
   #define COL6     117
   #define COL7     139

   /*
      Color definitions are custom properties of the container
      used to define the buttons:
        NormalColor is the normal color used if not selected
        SelectedColor is what it sounds like, so that the
           currently selected date stands out
        SundayColor is used to show the first column
           (Sunday) ... a lot of calendars do this, so 
           I figured I would, too ...
        HolidayColor is used to show holidays ...

           Note that holidays that are in the array defined
           by "LoadHolidays" (a method of the container class)
           will appear in this color as well, unless they 
           are the "selected" item ...
   */
   this.NORMALCOLOR   = "BLACK/BTNFACE"
   this.SELECTEDCOLOR = "RED/WHITE"
   this.SUNDAYCOLOR   = "MAROON/BTNFACE"
   this.HOLIDAYCOLOR  = "WHITE/NAVY"

   // instance of dateex class and holiday class:
   this.DateEx        = new DateEx()
   this.Holiday       = new Holiday()
   // used to set speedtips ...
   this.cSpeedTip     = ""
   // currentDate and beginDate:
   this.currentDate   = {}
   this.beginDate     = {}
   // Language -- hopefully for use with
   // the API calls in DateEx.cc, so we can
   // get the correct info for the month
   // and day of the week:
   this.Language      = "English (United States)"


   this.YEARMINUS10 = new PUSHBUTTON(this)
   with (this.YEARMINUS10)
      onClick     := class::YEARMINUS10_ONCLICK
      height      := 21
      left        := 11
      top         :=  4
      width       := 21
      text        := "10"
      fontSize    :=  8
      speedBar    := true
      speedTip    := "10 Years Back"
      colorNormal := "red/BtnFace"
   endwith

   this.PREVYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVYEARPUSHBUTTON)
      onClick  := CLASS::PREVYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 21
      width    := 18
      left     := 32
      top      := 4
      speedBar := true
      speedTip := "Previous Year"
   endwith

   this.TEXTYEAR = new TEXT(this)
   with (this.TEXTYEAR)
      fontSize        := 10
      text            := "9999"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      height          := 21
      left            := 49
      top             := 4
      width           := 73
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTYEARPUSHBUTTON)
      onClick  := CLASS::NEXTYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #855"
      text     := ""
      height   := 21
      width    := 18
      left     := 122
      top      := 4
      speedBar := true
      speedTip := "Next Year"
   endwith

   this.YEARPLUS10 = new PUSHBUTTON(this)
   with (this.YEARPLUS10)
      onClick     := class::YEARPLUS10_ONCLICK
      height      := 21
      left        := 141
      top         := 4
      width       := 21
      text        := "10"
      fontSize    := 8
      speedBar    := true
      speedTip    := "10 Years Ahead"
      colorNormal := "green/BtnFace"
   endwith

   this.MONTHMINUS3 = new PUSHBUTTON(this)
   with (this.MONTHMINUS3)
      onClick     := class::MONTHMINUS3_ONCLICK
      height      := 21
      left        := 11
      top         := 25
      width       := 21
      text        := "3"
      fontSize    := 8
      speedBar    := true
      speedTip    := "3 Months Back"
      colorNormal := "red/BtnFace"
   endwith

   this.PREVMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVMONTHPUSHBUTTON)
      onClick  := CLASS::PREVMONTHPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 21
      width    := 18
      left     := 32
      top      := 25
      speedBar := true
      speedTip := "Previous Month"
   endwith

   this.TEXTMONTH = new TEXT(this)
   with (this.TEXTMONTH)
      fontSize        := 10
      text            := "Month"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      height          := 21
      left            := 49
      width           := 73
      top             := 25
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTMONTHPUSHBUTTON)
      onClick  := CLASS::NEXTMONTHPUSHBUTTON_OnClick
      height   := 21
      width    := 18
      left     := 122
      top      := 25
      upBitmap := "RESOURCE #855"
      text     := ""
      speedBar := true
      speedTip := "Next Month"
   endwith

   this.MONTHPLUS3 = new PUSHBUTTON(this)
   with (this.MONTHPLUS3)
      onClick     := class::MONTHPLUS3_ONCLICK
      height      := 21
      left        := 141
      top         := 25
      width       := 21
      text        := "3"
      fontSize    := 8
      speedBar    := true
      speedTip    := "3 Months Ahead"
      colorNormal := "green/BtnFace"
   endwith

   this.TITLESUNDAY = new TitleText3(this)
   with (this.TITLESUNDAY)
      left := COL1
      top  := TITLEROW
   endwith

   this.TITLEMONDAY = new TitleText3(this)
   with (this.TITLEMONDAY)
      left := COL2
      top  := TITLEROW
      text := "M"
   endwith

   this.TITLETUESDAY = new TitleText3(this)
   with (this.TITLETUESDAY)
      left := COL3
      top  := TITLEROW
      text := "T"
   endwith


   this.TITLEWEDNESDAY = new TitleText3(this)
   with (this.TITLEWEDNESDAY)
      left := COL4
      top  := TITLEROW
      text := "W"
   endwith


   this.TITLETHURSDAY = new TitleText3(this)
   with (this.TITLETHURSDAY)
      left := COL5
      top  := TITLEROW
      text := "T"
   endwith


   this.TITLEFRIDAY = new TitleText3(this)
   with (this.TITLEFRIDAY)
      left := COL6
      top  := TITLEROW
      text := "F"
   endwith


   this.TITLESATURDAY = new TitleText3(this)
   with (this.TITLESATURDAY)
      left := COL7
      top  := TITLEROW
   endwith

   this.ROW1COL1 = new DayPushButton3(this)
   with (this.ROW1COL1)
      left = COL1
      top = ROW1
   endwith

   this.ROW1COL2 = new DayPushButton3(this)
   with (this.ROW1COL2)
      left = COL2
      top = ROW1
   endwith

   this.ROW1COL3 = new DayPushButton3(this)
   with (this.ROW1COL3)
      left = COL3
      top = ROW1
   endwith

   this.ROW1COL4 = new DayPushButton3(this)
   with (this.ROW1COL4)
      left = COL4
      top = ROW1
   endwith

   this.ROW1COL5 = new DayPushButton3(this)
   with (this.ROW1COL5)
      left = COL5
      top = ROW1
   endwith

   this.ROW1COL6 = new DayPushButton3(this)
   with (this.ROW1COL6)
      left = COL6
      top = ROW1
   endwith

   this.ROW1COL7 = new DayPushButton3(this)
   with (this.ROW1COL7)
      left = COL7
      top = ROW1
   endwith

   this.ROW2COL1 = new DayPushButton3(this)
   with (this.ROW2COL1)
      left = COL1
      top = ROW2
   endwith

   this.ROW2COL2 = new DayPushButton3(this)
   with (this.ROW2COL2)
      left = COL2
      top = ROW2
   endwith

   this.ROW2COL3 = new DayPushButton3(this)
   with (this.ROW2COL3)
      left = COL3
      top = ROW2
   endwith

   this.ROW2COL4 = new DayPushButton3(this)
   with (this.ROW2COL4)
      left = COL4
      top = ROW2
   endwith

   this.ROW2COL5 = new DayPushButton3(this)
   with (this.ROW2COL5)
      left = COL5
      top = ROW2
   endwith

   this.ROW2COL6 = new DayPushButton3(this)
   with (this.ROW2COL6)
      left = COL6
      top = ROW2
   endwith

   this.ROW2COL7 = new DayPushButton3(this)
   with (this.ROW2COL7)
      left = COL7
      top = ROW2
   endwith

   this.ROW3COL1 = new DayPushButton3(this)
   with (this.ROW3COL1)
      left = COL1
      top = ROW3
   endwith

   this.ROW3COL2 = new DayPushButton3(this)
   with (this.ROW3COL2)
      left = COL2
      top = ROW3
   endwith

   this.ROW3COL3 = new DayPushButton3(this)
   with (this.ROW3COL3)
      left = COL3
      top = ROW3
   endwith

   this.ROW3COL4 = new DayPushButton3(this)
   with (this.ROW3COL4)
      left = COL4
      top = ROW3
   endwith

   this.ROW3COL5 = new DayPushButton3(this)
   with (this.ROW3COL5)
      left = COL5
      top = ROW3
   endwith

   this.ROW3COL6 = new DayPushButton3(this)
   with (this.ROW3COL6)
      left = COL6
      top = ROW3
   endwith

   this.ROW3COL7 = new DayPushButton3(this)
   with (this.ROW3COL7)
      left = COL7
      top = ROW3
   endwith

   this.ROW4COL1 = new DayPushButton3(this)
   with (this.ROW4COL1)
      left = COL1
      top = ROW4
   endwith

   this.ROW4COL2 = new DayPushButton3(this)
   with (this.ROW4COL2)
      left = COL2
      top = ROW4
   endwith

   this.ROW4COL3 = new DayPushButton3(this)
   with (this.ROW4COL3)
      left = COL3
      top = ROW4
   endwith

   this.ROW4COL4 = new DayPushButton3(this)
   with (this.ROW4COL4)
      left = COL4
      top = ROW4
   endwith

   this.ROW4COL5 = new DayPushButton3(this)
   with (this.ROW4COL5)
      left = COL5
      top = ROW4
   endwith

   this.ROW4COL6 = new DayPushButton3(this)
   with (this.ROW4COL6)
      left = COL6
      top = ROW4
   endwith

   this.ROW4COL7 = new DayPushButton3(this)
   with (this.ROW4COL7)
      left = COL7
      top = ROW4
   endwith

   this.ROW5COL1 = new DayPushButton3(this)
   with (this.ROW5COL1)
      left = COL1
      top = ROW5
   endwith

   this.ROW5COL2 = new DayPushButton3(this)
   with (this.ROW5COL2)
      left = COL2
      top = ROW5
   endwith

   this.ROW5COL3 = new DayPushButton3(this)
   with (this.ROW5COL3)
      left = COL3
      top = ROW5
   endwith

   this.ROW5COL4 = new DayPushButton3(this)
   with (this.ROW5COL4)
      left = COL4
      top = ROW5
   endwith

   this.ROW5COL5 = new DayPushButton3(this)
   with (this.ROW5COL5)
      left = COL5
      top = ROW5
   endwith

   this.ROW5COL6 = new DayPushButton3(this)
   with (this.ROW5COL6)
      left = COL6
      top = ROW5
   endwith

   this.ROW5COL7 = new DayPushButton3(this)
   with (this.ROW5COL7)
      left = COL7
      top = ROW5
   endwith

   this.ROW6COL1 = new DayPushButton3(this)
   with (this.ROW6COL1)
      left = COL1
      top = ROW6
   endwith

   this.ROW6COL2 = new DayPushButton3(this)
   with (this.ROW6COL2)
      left = COl2
      top = ROW6
   endwith

   this.ROW6COL3 = new DayPushButton3(this)
   with (this.ROW6COL3)
      left = COL3
      top = ROW6
   endwith

   this.ROW6COL4 = new DayPushButton3(this)
   with (this.ROW6COL4)
      left = COL4
      top = ROW6
   endwith

   this.ROW6COL5 = new DayPushButton3(this)
   with (this.ROW6COL5)
      left = COL5
      top = ROW6
   endwith

   this.ROW6COL6 = new DayPushButton3(this)
   with (this.ROW6COL6)
      left = COL6
      top = ROW6
   endwith

   this.ROW6COL7 = new DayPushButton3(this)
   with (this.ROW6COL7)
      left = COL7
      top = ROW6
   endwith

   this.TodayPushButton = new PUSHBUTTON(this)
   with (this.TodayPushButton)
      onClick  = CLASS::TodayPushButton_OnClick
      height   = 18
      left     = 58
      top      = 208
      width    = 70
      text     = "Today"
      upBitmap = "RESOURCE #855"
      fontSize = 10
      speedTip = "Go to Today's Date"
   endwith

   function CheckMetric
      // called from container's onDesignOpen:
      if form.metric # 6
         msgbox( "Form's metric property is not '6 - Pixels', "+;
                 "this will not display properly.", ;
                 "Bad Form Metric!", 16 )
      endif

   function containerOpen
      // default date if needed:
      if type( 'this.currentDate' ) # "D" or ; 
         empty( this.currentDate )
         this.currentDate = date()
      endif

      // Set color of "sunday" title button
      this.titleSunday.colorNormal := this.SUNDAYCOLOR

      // store starting date value, in
      // case there's a cancel option,
      // the user can return to the 'beginDate':
      this.beginDate = this.currentDate

      #if __version__ > 2.5
         this.YearMinus10.systemTheme         := false
         this.YearPlus10.systemTheme          := false
         this.MonthMinus3.systemTheme         := false
         this.MonthPlus3.systemTheme          := false
         this.NextMonthPushbutton.systemTheme := false
         this.PrevMonthPushbutton.systemTheme := false
         this.NextYearPushbutton.systemTheme  := false
         this.PrevYearPushbutton.systemTheme  := false
      #endif

      // update day-of-week headings
      class::DayHeadings()

      // newMonth routine
      class::NewMonth()
   return

   function DayHeadings
      // deal with language stuff for the
      // day of the week:
      local cDay, dTemp

      // Set the language for the DateEx object:
      this.dateEx.Language := this.Language

      // we need a day known to be a Sunday:
      dTemp = new date( 2004, 1, 1 ) // February 1, 2004
                                     // remember date object
                                     // is zero-based, hence
                                     // the second month of
                                     // the year is 1
      // Get Sunday
      cDay = this.dateEx.intlcdow( dTemp )
      // Assign first letter ...:
      this.TitleSunday.text := left( cDay, 1 )
      // Get Monday
      cDay = this.dateEx.intlcdow( dTemp+1 )
      // Assign first letter ...:
      this.TitleMonday.text := left( cDay, 1 )
      // Get Tuesday
      cDay = this.dateEx.intlcdow( dTemp+2 )
      // Assign first letter ...:
      this.TitleTuesday.text := left( cDay, 1 )
      // Get Wednesday
      cDay = this.dateEx.intlcdow( dTemp+3 )
      // Assign first letter ...:
      this.TitleWednesday.text := left( cDay, 1 )
      // Get Thursday
      cDay = this.dateEx.intlcdow( dTemp+4 )
      // Assign first letter ...:
      this.TitleThursday.text := left( cDay, 1 )
      // Get Friday
      cDay = this.dateEx.intlcdow( dTemp+5 )
      // Assign first letter ...:
      this.TitleFriday.text := left( cDay, 1 )
      // Get Saturday
      cDay = this.dateEx.intlcdow( dTemp+6 )
      // Assign first letter ...:
      this.TitleSaturday.text := left( cDay, 1 )
   return


   function newMonth
      // this function deals with setting values based
      // on changes to month/year -- determine where 
      // first day is, last day is, everything in 
      // between: set button texts, colors ...

      // First: need to determine what level we're at --
      // are we working from container, or control
      // on container (cRef needs to point to the container):
      if "CONTAINER" $ upper( this.className ) or;
         "CONTAINER" $ upper( this.baseClassName )
         // we're on the container
         oRef = this  
      else
         // we're on a pushbutton _on_ the container
         oRef = this.parent 
      endif

      // Make sure holiday list is current ...
      class::LoadHolidays()

      // now to get first day of month and last day of
      // month from 'currentDate'
      dDate  = oRef.currentDate
      dFirst = oRef.dateEx.fdom( dDate )
      dLast  = oRef.dateEx.ldom( dDate )
      nDay   = day( dDate ) // current day of month

      // character values for month and year:
      oRef.textMonth.text := oRef.dateEx.intlCMon( dDate )
      oRef.textYear.text  := ltrim( str( year( dDate ) ) )

      // now the fun part ... assigning values, based
      // on where we start:
      nStart = dow( oRef.currentDate )

      // Clear out any outstanding speedTips
      for nRows = 1 to 6
          for nColumns = 1 to 7
              cObject = "oRef.Row1Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip. := ""
          next
      next

      // We need to loop through everything, and set
      // visible properties as well as text if it's going
      // to display:
      nCurrent = 0
      for nRows = 1 to 6
          for nColumns = 1 to 7
              nCurrent++
              // start at day 1, but ...
              // for first row, loop through until we find
              // the first one (and make everything before
              // the first day of month invisible):
              if nCurrent == 1
                 do while nCurrent <= nColumns
                    if dow( dFirst ) == nColumns
                       // set the text:
                       cObject = "oRef.Row1Col"+nColumns
                       cText   = cObject+".text"
                       &cText. := "1"
                       // Set colors:
                       cColor = cObject+".colorNormal"
                       // if the day we're setting is the
                       // selected/current date:
                       dTestDate = new date(year(dDate), ;
                                   month(dDate)-1, nCurrent)
                       // Make it a real date
                       dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                                   iif( set("CENTURY") == "ON", 10, 8 ) ) )
                       if nDay == nCurrent
                          &cColor. := oRef.SELECTEDCOLOR
                          oRef.currentButton = &cObject.
                          cSpeedTip = cObject+".speedTip"
                          if class::isHoliday( dtestDate )
                             &cSpeedTip. = oRef.cSpeedTip
                          endif                       
                       else
                          // is it a sunday or a holiday?
                          cSpeedTip = cObject+".speedTip"
                          if nColumns == 1 
                             &cColor. := oRef.SUNDAYCOLOR
                             if class::IsHoliday( dTestDate )
                                &cColor.    := oRef.HOLIDAYCOLOR
                                &cSpeedTip. := oRef.cSpeedTip
                             endif
                          elseif class::IsHoliday( dTestDate )
                             &cColor. := oRef.HOLIDAYCOLOR
                             &cSpeedTip. := oRef.cSpeedTip
                          else
                             // it's any other day of week ...
                             &cColor. := oRef.NORMALCOLOR
                          endif
                       endif // nDay == nCurrent

                       // set visible property
                       cVisible = cObject+".visible"
                       &cVisible. := true
                       // finish the row
                       for i = nColumns to 7
                           cObject = "oRef.Row1Col"+i
                           cText   = cObject+".text"
                           &cText. := "1"
                           // set visible property
                           cVisible = cObject+".visible"
                           &cVisible. := true
                       next
                       exit // out of this loop, we're done
                    else 
                       if nColumns > 7
                          nColumns = 7
                          exit
                       endif
                       cObject = "oRef.Row1Col"+nColumns
                       cVisible = cObject+".visible"
                       &cVisible. := false
                    endif // dow( dFirst ) = nColumns
                    nColumns++ // increment column counter
                    loop       // top of _this_ loop
                 enddo // while nCurrent <= nColumns
                 loop // back to columns for/next loop
              endif // nCurrent = 1

              // now to attempt to handle setting
              // text and colors ...
              cObject = "oRef.Row"+nRows+"Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip := "" // clear out whatever's there
              // if we're inside the month's dates:
              if nCurrent > 1 and nCurrent <= day( dLast )
                 // set the text property:
                 cText = cObject+".text"
                 &cText. := ""+nCurrent
                 // set the button to visible:
                 cVisible = cObject+".visible"
                 &cVisible. := true
                 // deal with colors:
                 cColor = cObject+".colorNormal"
                 // if the day we're setting is the
                 // selected/current date:
                 dTestDate = new date(year(dDate), ;
                             month(dDate)-1, nCurrent)
                 // Make it a real date
                 dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                             iif( set("CENTURY") == "ON", 10, 8 ) ) )
                 if nDay == nCurrent
                    &cColor. := oRef.SELECTEDCOLOR
                    oRef.currentButton = &cObject.
                    cSpeedTip = cObject+".speedTip"
                    if class::isHoliday( dtestDate )
                       &cSpeedTip. = oRef.cSpeedTip
                    endif                       
                 else
                    // is it a sunday or a holiday?
                    cSpeedTip = cObject+".speedTip"
                    if nColumns == 1 
                       &cColor. := oRef.SUNDAYCOLOR
                       if class::IsHoliday( dTestDate )
                          &cColor.    := oRef.HOLIDAYCOLOR
                          &cSpeedTip. := oRef.cSpeedTip
                       endif
                    elseif class::IsHoliday( dTestDate )
                       &cColor. := oRef.HOLIDAYCOLOR
                       &cSpeedTip. := oRef.cSpeedTip
                    else
                       // it's any other day of week ...
                       &cColor. := oRef.NORMALCOLOR
                    endif
                 endif // nDay == nCurrent
              else
                 // we want to make it invisible
                 cVisible = cObject+".visible"
                 &cVisible. := false
              endif // nCurrent > 1 and nCurrent <= day( dLast )
          next // nColumns
       next // nRows

   function TodayPushButton_onClick
      // set today's date
      this.parent.currentDate = date()
      class::NewMonth()
   return

   function NEXTYEARPUSHBUTTON_onClick
      // code to add year to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, 1 )
      class::NewMonth()      
   return

   function YEARPLUS10_onClick
      // code to add 10 years to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, 10 )
      class::NewMonth()      
   return

   function YEARMINUS10_onClick
      // code to subtract 10 years from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, -10 )
      class::NewMonth()      
   return

   function NEXTMONTHPUSHBUTTON_onClick
      // code to add month to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, 1 )
      class::NewMonth()
   return

   function MONTHPLUS3_onClick
      // code to add 3 months to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, 3 )
      class::NewMonth()
   return

   function PREVMONTHPUSHBUTTON_onClick
      // code to subtract month from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, -1 )
      class::NewMonth()     
   return

   function MONTHMINUS3_onClick
      // code to subtract 3 months from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, -3 )
      class::NewMonth()     
   return

   function PREVYEARPUSHBUTTON_onClick
      // code to subtract year from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, -1 )
      class::NewMonth()
   return

   procedure LoadHolidays
      // Creates/re-creates array used to determine
      // a holiday ... NOTE: It adds an array
      // to the form ...
      local nYear, oRef, nHoliday, d
      local cMarkSet
      private cCmd, cDateSet


      // Have to determine if being called from container
      // or from a button call ... 
      if "CALENDAR" $ this.className
         nYear = year( this.currentDate )
         oRef = this
      else
         nYear = year( this.parent.currentDate )
         oRef = this.parent
      endif

      // Create array
      oRef.Holidays = new Array(1,2) // 1 row, 2 columns
      
      // Load holidays using holidaydate method below:
      // floating holidays (most of these are US Holidays --
      // *********************************************
      // you may wish to modify if not in the US ...):
      // *********************************************
      // President's Day
      i = 0
      i++
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "P" )
      oRef.Holidays[i,2] = "President's Day"
      // Daylight Savings time -- always a Sunday
      // Memorial Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "M" )
      oRef.Holidays[i,2] = "Memorial Day"
      // Labor Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "L" )
      oRef.Holidays[i,2] = "Labor Day"
      // Columbus Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "C" )
      oRef.Holidays[i,2] = "Columbus Day"
      // Standard Time -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "S" )
      oRef.Holidays[i,2] = "Standard Time"
      // Daylight Savings Time
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "D" )
      oRef.Holidays[i,2] = "Daylight Savings Time"
      // Election Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "E" )
      oRef.Holidays[i,2] = "Election Day"
      // Thanksgiving
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "T" )
      oRef.Holidays[i,2] = "Thanksgiving"
      // Advent -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "A" )
      oRef.Holidays[i,2] = "Advent"

      // Easter requires special code ...:
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.EasterDay( nYear )
      oRef.Holidays[i,2] = "Easter"

      // fixed dates:
      // deal with date format, so we don't
      // mess up international users ...
      cDateSet = set("DATE")
      cMarkSet = set("MARK")
      set date MDY

      // New Year's Day
      d = ctod( "01/01/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Day"
      // Lincoln's Birthday
      d = ctod( "02/12/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Lincoln's Birthday"
      // Valentine's Day
      d = ctod( "02/14/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Valentine's Day"
      // Washington's Birthday
      d = ctod( "02/22/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Washington's Birthday"
      // St. Patrick's Day
      d = ctod( "03/17/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "St. Patrick's Day"
      // April Fool's Day
      d = ctod( "04/01/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "April Fool's Day"
      // Fourth of July
      d = ctod( "07/04/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Independance Day"
      // Halloween (All Hallows Eve)
      d = ctod( "10/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Halloween"
      // Christmas
      d = ctod( "12/25/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Christmas"
      // New Year's Eve
      d = ctod( "12/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Eve"

      // You can add more here if you wish ...
      // make sure you add a row to the array, 
      // and add the name of the holiday or memorable
      // date ... Basically, copy the code as shown
      // above, and make appropriate changes

      // reset date format to whatever it was
      // before we changed to American
      set date &cDateSet.
      cCmd = "set mark to '"+cMarkSet+"'"
      &cCmd.

      // one last thing:
      oRef.Holidays.sort(1) // sort on first column

   Function IsHoliday
      parameter dCheckIt
      // this routine reads the array "this.Holidays"
      // and checks the date to see if the date in
      // dCheck parameter is in the list, and returns
      // a logical value ... 
      if "CALENDAR" $ upper( this.className )
         oRef = this
      else
         oRef = this.parent
      endif
      lHoliday = false
      oRef.cSpeedTip = ""
      for i = 1 to oRef.Holidays.size/2
          if oRef.Holidays[ i, 1 ] == dCheckIt
             lHoliday := true
             oRef.cSpeedTip = oRef.Holidays[ i, 2 ]
          endif
      next
   return ( lHoliday )

endclass
// end of class pKenCalendarContainer

/* 
   ----------------------------------------------------------
   Pushbutton class definition:
   ----------------------------------------------------------
*/
class DayPushButton3( oParent ) of PushButton( oParent )
   with (this)
      fontSize := 12
      height   := 22
      width    := 22
      metric   := 6  // Pixels
      text     := "30"
      pageno   := 0
      visible  := false
      speedBar := true // so they don't keep focus
   endwith
   #if __version__ > 2.5
      // for these toolbuttons, don't use the system theme ...
      this.systemTheme := false // for dBASE Plus 2.6 or later
   #endif

   function onClick
      /*
         The tricky part of this little routine is
         resetting colors -- in order to do that,
         we need to store a reference to the 'current'
         button, so we can reset that, before we change
         the now current button's color ...
      */
      dCurrentDate = new date(year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.parent.currentButton.text))
      dCurrentDate = ctod( left( dCurrentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )
      if type( "this.parent.currentbutton" ) # "U"
         // if it's in Column 1, it's a "sunday" ...
         if "COL1" $ this.parent.currentButton.Name 
            this.parent.currentButton.colorNormal := ;
                                  this.parent.SUNDAYCOLOR
         // set holiday color if needed:
         elseif this.parent.isHoliday( dCurrentDate )
               this.parent.currentButton.colorNormal := ;
                                  this.parent.HOLIDAYCOLOR
         // otherwise, we have a "normal" day ...
         else
            this.parent.currentButton.colorNormal := ;
                                    this.parent.NORMALCOLOR
         endif
      endif

      // Assign currentButton property of container
      this.parent.currentButton = this
      // Assign color ...
      this.colorNormal := this.parent.SELECTEDCOLOR

      // set current date to this one:
      this.parent.currentDate := ;
           new date( year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.text) )
      this.parent.currentDate := ;
                     ctod( left( this.parent.currentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )

endclass

// custom class the "title" text so we have something nearly
// identical to the pushbuttons used for the days for the titles
class TitleText3( oParent ) of Text( oParent )
   with (this)
      fontSize        := 12
      height          := 22
      width           := 22
      metric          := 6  // Pixels
      text            := "S"
      pageno          := 0
      border          := true
      borderStyle     := 1 // Raised
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
   endwith

endclass

/*
   ================================================================
   End of file: KenCal3.CC
   ================================================================
*/




