// DEPENDENCIES:
// :dUFLP:rmRegistry.cc
//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// Non visual class object line
class rmBDEconfig
                               
// Visual form object class line
// class rmBDEconfig(oP) of PaintBox(oP) custom 

   this.version = "1.06.0416"

   // File dependency
   if "RUNTIME" $ upper(version(1)) == true
      set procedure to rmRegistry.cc additive
   else
      try
         set procedure to :dUFLP:rmRegistry.cc additive
      catch (exception e)
         try
            set procedure to rmRegistry.cc additive
         catch (exception e)
            msgbox("Required file - rmRegistry.cc not found." + chr(13) ;
                 + chr(13) ;
                 + "Either the source alias to this file does not exist," ;
                 + chr(13) ;
                 + "or the file is not found in the current directory." ;
                 , " Error", 16 +0)
         endtry
      endtry
   endif


   // *********************************************************************
   // Language Selection <<---  <<---  <<---  <<---  <<---  <<---  LANGUAGE
   //                                                              ********

   #define RMLANGUAGE "English"
   // #define RMLANGUAGE "Italian"
   // #define RMLANGUAGE "German"
   // #define RMLANGUAGE "Spanish"
   // #define RMLANGUAGE "Danish"
   // #define RMLANGUAGE "French"
   // #define RMLANGUAGE "Traditional Chinese"

   //                                                              ********
   // Language Selection <<---  <<---  <<---  <<---  <<---  <<---  LANGUAGE
   // *********************************************************************

   /*

   rmBDEconfig.cc 

   A custom control which provides the ability to configure the BDE from your
   application, and automatically check that those settings are still in
   existence every time your application starts up. If a change is required an
   update is initiated, with an easily configurable level of user interaction
   and notification.

   This control can also be used as a visual form object by editing the class
   definition line at the very top.


   For         :  dBASE Plus 2.5 onwards / Win200* / XP


   Author      :  Ronnie MacGregor


   Contact     :  rmBDEconfigFeedback@dBASEdeveloper.co.uk

   Copyright   :  Ronnie MacGregor   All rights reserved.

   Disclaimer  :  It is the duty of the user of this code to determine its 
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.
                  
                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.



   Acknowledgements :

      Jan Hoelterling - Elements of the approach Jan took with ensuring
                        the best BDE settings are implemented in this cc.
                        Jan's code came with the recommendation of
                        Romain Strieff so that had to be a good
                        starting point <g>.

      Ken Mayer       - Output from BDERegistrySettings.wfm (dUFLP) used as a
                        guide for BDE settings (alongside regedit).
                                 
      Vic McClung     - BDE restart code

      Jan Hoelterling - Beta testing, bug reporting and suggestions.



   Translation :

      Simone Bartoccioni - Italian
      Franco Asioli      - Italian
      Jan Hoelterling    - German
      Heinz Kesting      - German
      Emilio Vilaro      - Spanish
      Ivar B. Jessen     - Danish
      Marc Hamelin       - French
      Jean-Pierre Martel - French
      Michael NG         - Traditional Chinese



   Properties (Those properties intended to be set by the developer):


      this.AskUser = false

         Whether to ask the user before making any changes to BDE settings.


      this.TellUserChanged = false

         Whether to tell the user the detail of what has been updated.


      this.TellUserNoChange = false

         Whether to tell the user if no changes made.


      this.SilentRestart = false

         Whether the application should just shut down and restart automatically
         with no user interaction. Intended to be used in compiled distributed
         applications. Read the OLH on _app.EXEName before you use this option
         to ensure you get the expected behaviour.
         

      this.BDErestart = true

         Whether to restart the BDE after an update.


      this.LogDir = ""

         The location of the log file if this is to be anything other than the
         default location. The default location is the same directory that
         this control is run from. This is intended to allow several
         applications running from different locations to update a single copy
         of the log file therefore listing all BDE changes. This can be anywhere
         on the system where the user has create and write permissions.


      this.RuntimeLogOnly = false 

         Whether to only update the log file when being run in the runtime
         environment.



   Methods :

      Update_BDE_*(bFinalise)

         Updates the BDE settings for that section to the associated control
         custom property values, which should be set first.

         bFinalise indicates whether you wish to finish up after calling this
         function (true), or whether you have more to do (false).


      Update_DbaseNativeDriver(bFinalise)

         Updates the settings for the dBASE native driver to the associated control
         custom property values, which should be set first.

         bFinalise indicates whether you wish to finish up after calling this
         function (true), or whether you have more to do (false).


      Update_MSSQLNativeDriver(bFinalise)

         Updates the settings for the MSSQL native driver to the associated control
         custom property values, which should be set first.

         bFinalise indicates whether you wish to finish up after calling this
         function (true), or whether you have more to do (false).


      UpdateAll()

         Updates all sections to the associated control custom property values,
         which should be set first. It is anticipated that in real world use
         the default properties in this custom class would be edited to suit
         the needs of the developer, allowing a single call to this function to
         do the whole job.



   Usage :

      **************************************************************************
      IMPORTANT  This control will overwrite existing BDE settings if they differ
      *********  from the control custom property values in any way.
                 You must ensure that the control default settings are
                 acceptable to you, or are being set by you to suit.
                 You must understand what this control will do before you use it.
                 No responsibility will be accepted for any problems arising
                 from the use of this control.
                 It is your duty to ensure that it does what you expect before
                 "going live".
      **************************************************************************

      Check all the cc default settings to ensure that they meet your requirements.

      Hard code any changes to those settings you wish.

      Add this code to the onOpen code for the first form in your application.


         set procedure to rmBDEConfig.cc additive

         local o
         o = new RMBDECONFIG()
         o.UpdateAll()
         release object o
         o = null


      See rmBDEConfigTest.wfm for a test form.

      See rmBDEConfigSample.wfm for a "real world" example of usage.


      If you want to use this class as a visual form class then edit the class
      definition line commenting at the top of this file, and then as the first
      control in z order you can do :

         set procedure to rmBDEConfig.cc additive

         this.rmBDEConfig = new rmBDEConfig(this)
         with (this.rmBDEConfig)
            onOpen = {; this.UpdateAll()}
            height = 20.0
            left = 5.0
            top = 5.0
            width = 20.0
         endwith



   Language Support :

      Language support is built in to this control. Simply uncomment the define
      statement RMLANGUAGE for your chosen language.
      


   Notes :

      Having asked for opinion in the newsgroups, it seems that the Init section
      settings will propogate to the Native Driver settings for other database
      types, such that a specific function in this control is probably not required.

      If you have experience of other database types, and would like to see
      specific settings added to this control please contact me by email.



   Revisions :

      None


   */


//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

   // Universal Constants

   // Line Break
   #define CR                   chr(13)+chr(10)

   // Registry Main Key
   #define HKEY_LOCAL_MACHINE   (0x80000002)

   // Define first part of subkey
   #define BDE_REG_KEY          "SOFTWARE\Borland\Database Engine"

   // Registry Constants
   #define ERROR_SUCCESS        0
   #define ERROR_ACCESS_DENIED  5

   // BDE Constants
   #define DBIERR_NONE          0



   // English Language Constants

   #if RMLANGUAGE == "English"

      // Ask the user dialogue
      #define ASK_TITLE            " BDE Update Required"
      #define ASK_EXPLAIN          "The BDE (Borland Database Engine) needs a settings update in order to ensure correct functionality."
      #define ASK_QUESTION         "Would you like to perform this update now ?"
      #define ASK_ADVICE           "' Yes ' - recommended"

      // Report dialogue
      #define TELL_TITLE           " BDE Update Information"
      #define TELL_EXPLAIN         "The following changes were made to the BDE :"
      #define TELL_SETTING         "Setting"
      #define TELL_CHANGED         "changed"
      #define TELL_FROM            "from"
      #define TELL_TO              "to"
      #define TELL_NO_CHANGE       "No changes to BDE settings were made."

      // BDE Restart dialogue
      #define RESTART_TITLE        " BDE Update Information"
      #define RESTART_EXPLAIN_1    "The BDE (Borland Database Engine) has had its settings updated."
      #define RESTART_EXPLAIN_2    "This application will now be closed in order for these changes to take effect."
      #define RESTART_EXPLAIN_3    "Please restart this application."

      // Error Dialogue
      #define ERROR_TITLE          " Registry Update Error"
      #define ERROR_EXPLAIN        "Error writing registry value"

      // User rights
      #define RIGHTS_EXPLAIN_1     "You do not have administrator rights."                     
      #define RIGHTS_EXPLAIN_2     "Administrator rights are required to perform this task."   
      #define RIGHTS_EXPLAIN_3     "Please log on with administrator rights and run this application again in order to perform update."

   #endif


   // Italian Language Constants
   // Thanks to Simone Bartoccioni and Franco Asioli for translation

   #if RMLANGUAGE == "Italian"

      // Ask the user dialogue
      #define ASK_TITLE            " Aggiornamento BDE richiesto"
      #define ASK_EXPLAIN          "Il BDE (Borland Database Engine) necessit di un aggiornamento della configurazione per funzionare correttamente."
      #define ASK_QUESTION         "Vuoi effettuare l'aggiornamento ora ?"
      #define ASK_ADVICE           "' Si ' - raccomandato"

      // Report dialogue
      #define TELL_TITLE           " Informazioni aggiornamento BDE"
      #define TELL_EXPLAIN         "I seguenti cambiamenti sono stati apportati al BDE :"
      #define TELL_SETTING         "Impostazioni"
      #define TELL_CHANGED         "variate"
      #define TELL_FROM            "da"
      #define TELL_TO              "a"
      #define TELL_NO_CHANGE       "Non  stato effettuato nessun cambiamento ai settaggi del BDE."

      // BDE Restart dialogue
      #define RESTART_TITLE        " Informazioni aggiornamento BDE"
      #define RESTART_EXPLAIN_1    "I settaggi del BDE (Borland Database Engine) sono stati aggiornati."
      #define RESTART_EXPLAIN_2    "Questa applicazione verr ora chiusa in modo che le variazioni abbiano effetto."
      #define RESTART_EXPLAIN_3    "Per favore, riavviare questa applicazione."

      // Error Dialogue
      #define ERROR_TITLE          " Errore aggiornamento del registro"
      #define ERROR_EXPLAIN        "Errore scrittura valore nel registro"

      // User rights
      #define RIGHTS_EXPLAIN_1     "Non si hanno i privilegi di amministratore."
      #define RIGHTS_EXPLAIN_2     "I privilegi di amministratore sono richiesti per effettuare questa operazione."
      #define RIGHTS_EXPLAIN_3     "Per favore, accedere con i privilegi di amministratore ed eseguire nuovamente questa applicazione per effettuare l'aggiornamento."



   #endif


   // German Language Constants
   // Thanks to Jan Hoelterling and Heinz Kesting for translation

   #if RMLANGUAGE == "German"

      // Ask the user dialogue
      #define ASK_TITLE            " BDE Update ntig"
      #define ASK_EXPLAIN          "Um eine einwandfreie Programmfunktion zu gewhrleisten, mssen Einstellungen an der BDE (Borland Datenbank Engine) vorgenommen werden."
      #define ASK_QUESTION         "Die Einstellungen jetzt anpassen ?"
      #define ASK_ADVICE           "' Ja ' - empfohlen"

      // Report dialogue
      #define TELL_TITLE           " BDE Update Information"
      #define TELL_EXPLAIN         "Die folgenden Einstellungen der BDE wurden gendert :"
      #define TELL_SETTING         "Einstellung"
      #define TELL_CHANGED         "gendert"
      #define TELL_FROM            "von"
      #define TELL_TO              "auf"
      #define TELL_NO_CHANGE       "Die BDE Einstellungen wurden nicht gendert."

      // BDE Restart dialogue
      #define RESTART_TITLE        " BDE Update Information"
      #define RESTART_EXPLAIN_1    "Die Einstellungen der BDE (Borland Databank Engine) wurden gendert."
      #define RESTART_EXPLAIN_2    "Diese Anwendung muss schliessen, damit diese nderungen wirksam werden."
      #define RESTART_EXPLAIN_3    "Bitte starten Sie die Anwendung neu."

      // Error Dialogue
      #define ERROR_TITLE          " Schreibfehler Registry"
      #define ERROR_EXPLAIN        "Fehler beim Schreiben der Registry"

      // User rights
      #define RIGHTS_EXPLAIN_1     "Sie besitzen keine Administrator Rechte."
      #define RIGHTS_EXPLAIN_2     "Um diesen Vorgang durchfhren zu knnen, sind Administrator Rechte erforderlich."  
      #define RIGHTS_EXPLAIN_3     "Um das Update auszufhren, melden Sie sich bitte mit Administrator Rechten an und starten diese Anwendung erneut."

   #endif


   // Spanish Language Constants
   // Thanks to Emilio Vilaro for translation

   #if RMLANGUAGE == "Spanish"

      // Ask the user dialogue
      #define ASK_TITLE            " Se necesita la actualizacin del BDE"
      #define ASK_EXPLAIN          "El Motor BDE de Borland necesita la actualizacin de un parametro para poder funcionar correctamente."
      #define ASK_QUESTION         " Desea realizar esta actualizacin ahora ?"
      #define ASK_ADVICE           "' S ' - se recomienda hacerlo"

      // Report dialogue
      #define TELL_TITLE           " Informacin de la Actualizacin del BDE"
      #define TELL_EXPLAIN         "Los siguientes cambios se han realizado en el BDE :"
      #define TELL_SETTING         "Cambi de" // change of  // These constants "misused"
      #define TELL_CHANGED         "valor"     // value      // in order to read correctly
      #define TELL_FROM            "de"
      #define TELL_TO              "a"
      #define TELL_NO_CHANGE       "No se realizaron cambios en el BDE"

      // BDE Restart dialogue
      #define RESTART_TITLE        " Informacin de la Actualizacin del BDE"
      #define RESTART_EXPLAIN_1    "El Motor BDE de Borland ha realizado las actualizaciones."
      #define RESTART_EXPLAIN_2    "Esta aplicacin se cerrar, para que estos cambios se activen"
      #define RESTART_EXPLAIN_3    "Por favor reinicie la aplicacin."

      // Error Dialogue
      #define ERROR_TITLE          " Error al actualizar el Registro"
      #define ERROR_EXPLAIN        "Error al escribir un valor en el registro"

      // User rights
      #define RIGHTS_EXPLAIN_1     "Usted no tiene derechos como administrador."
      #define RIGHTS_EXPLAIN_2     "Se precisan derechos como administrador para realizar esta tarea."
      #define RIGHTS_EXPLAIN_3     "Por favor, entre como administrador y ejecute esta aplicacin otra vez para realizar la actualizacin."

   #endif


   // Danish Language Constants
   // Thanks to Ivar B. Jessen for translation

   #if RMLANGUAGE == "Danish"

      // Ask the user dialogue
      #define ASK_TITLE            " Opdatering af BDE ndvendig"
      #define ASK_EXPLAIN          "Opdatering af indstillingerne i Borlands Database Engine (BDE) er ndvendig for at sikre korrekt virkemde."
      #define ASK_QUESTION         "nskes denne opdatering foretaget nu ?"
      #define ASK_ADVICE           "' Ja ' - anbefales"

      // Report dialogue
      #define TELL_TITLE           " BDE Opdateringsinformation"
      #define TELL_EXPLAIN         "Der blev foretaget flgende ndringer i BDE :"
      #define TELL_SETTING         "Indstilling"
      #define TELL_CHANGED         "ndret"
      #define TELL_FROM            "fra"
      #define TELL_TO              "til"
      #define TELL_NO_CHANGE       "Der blev ikke foretaget ndringer i BDE-indstillingerne."

      // BDE Restart dialogue
      #define RESTART_TITLE        " BDE Opdateringsinformation"
      #define RESTART_EXPLAIN_1    "Indstillingerne i Borlands Database Engine (BDE) er blevet opdateret."
      #define RESTART_EXPLAIN_2    "Programmet vil nu blive lukket for at stte disse ndringer i kraft."
      #define RESTART_EXPLAIN_3    "Venligst genstart programmet."

      // Error Dialogue
      #define ERROR_TITLE          " Fejl under opdatering af registret"
      #define ERROR_EXPLAIN        "Fejl under skrivning af registervrdi"

      // User rights
      #define RIGHTS_EXPLAIN_1     "De har ikke administratortilladelser."
      #define RIGHTS_EXPLAIN_2     "Der krves administratortilladelser for at udfre denne handling."  
      #define RIGHTS_EXPLAIN_3     "Log p med administratorrettighed og kr dette program igen for at udfre opdateringen."

   #endif


   // French Language Constants
   // Thanks to Marc Hamelin and Jean-Pierre Martel for translation

   #if RMLANGUAGE == "French"

      // Ask the user dialogue
      #define ASK_TITLE            " Mise  jour du BDE requise"
      #define ASK_EXPLAIN          "Les paramtres du BDE (Borland Database Engine) doivent tre mis  jour pour assurer le bon fonctionnement du logiciel."
      #define ASK_QUESTION         "Voulez-vous effectuer cette mise  jour maintenant ?"
      #define ASK_ADVICE           "' Oui ' - Recommand"

      // Report dialogue
      #define TELL_TITLE           " Information sur la mise  jour du BDE"
      #define TELL_EXPLAIN         "Les modification suivantes ont t effectues au BDE :"
      #define TELL_SETTING         "Paramtre"
      #define TELL_CHANGED         "modifi"
      #define TELL_FROM            "de"
      #define TELL_TO              ""
      #define TELL_NO_CHANGE       "Aucune modification n'a t apporte aux paramtres du BDE."

      // BDE Restart dialogue
      #define RESTART_TITLE        " Information sur la mise  jour du BDE"
      #define RESTART_EXPLAIN_1    "Les paramtres du BDE (Borland Database Engine) ont t mis  jour."
      #define RESTART_EXPLAIN_2    "Le logiciel sera maintenant arrt pour que ces changements puissent prendre effet."
      #define RESTART_EXPLAIN_3    "Veuillez redmarrer ce logiciel."

      // Error Dialogue
      #define ERROR_TITLE          " Erreur de mise  jour du registre"
      #define ERROR_EXPLAIN        "Une erreur d'criture est survenue lors d'un enregistrement au registre"

      // User rights
      #define RIGHTS_EXPLAIN_1     "Vous ne dtenez pas les privilges d'un administrateur."
      #define RIGHTS_EXPLAIN_2     "Les privilges d'administrateur sont requis pour excuter cette tche."
      #define RIGHTS_EXPLAIN_3     "Veuillez ouvrir une nouvelle session possdant les privilges Administrateur et excuter cette application de nouveau pour effectuer la mise  jour."

   #endif



   // Traditional Chinese
   // Thanks to Michael NG for translation

   #if RMLANGUAGE == "Traditional Chinese"

      // Ask the user dialogue
      #define ASK_TITLE            " nDs BDE" 
      #define ASK_EXPLAIN          "BDE (Borland Database Engine) ݭns]w~i`B@L~"
      #define ASK_QUESTION         "{bߧYs ?"
      #define ASK_ADVICE           "' O ' - ĳs"
 
      // Report dialogue
      #define TELL_TITLE           " BDE sT "
      #define TELL_EXPLAIN         "BDE w@XHU :"
      #define TELL_SETTING         "]w"
      #define TELL_CHANGED         "w"
      #define TELL_FROM            ""
      #define TELL_TO              ""
      #define TELL_NO_CHANGE       " BDE ]wå@X"

      // BDE Restart dialogue
      #define RESTART_TITLE        " BDE sT "
      #define RESTART_EXPLAIN_1    "BDE (Borland Database Engine) ws]w"
      #define RESTART_EXPLAIN_2    "Ϥwܪ]wͮġA{ǱN|"
      #define RESTART_EXPLAIN_3    "Эs_ʵ{"

      // Error Dialogue
      #define ERROR_TITLE          " nɧs~"
      #define ERROR_EXPLAIN        "gJnɮɵoͿ~"

      // User rights
      #define RIGHTS_EXPLAIN_1     "AS޲zv"                     
      #define RIGHTS_EXPLAIN_2     "oȻݭn޲zv"  
      #define RIGHTS_EXPLAIN_3     "ЭsH޲znJAùB{ǰs"

   #endif

   // -------------------------------------

   // API Externs

   if type('DbiExit') # "FP"
      extern cword DbiExit(;
         cvoid ;
         );
      idapi32.dll
   endif

   if type('DbiInitFn') # "FP"
      extern cword DbiInitFn(;
         cword, ;
         cptr ;
         );
      idapi32.dll
   endif

   // -------------------------------------

   // Stock properties
   try
      this.visible := false
      if form.metric == 6 // pixels
         this.width := 20
         this.height := 20
      endif
   catch(exception e)
      // Based on object class not paintbox
   endtry

   // -------------------------------------

   // Custom Properties

   // User interaction
   this.AskUser = false                    // Ask User before update
   this.UserAgrees = "Unknown"             // User agrees to update
   this.TellUserChanged = false            // Tell User what has been changed
   this.TellUserNoChange = false           // Tell User if no changes
   this.TellUserError = false              // Tell User if there is an error
   this.TellUserUpdateStillRequired = true // Tell User they need admin rights to perform update
   this.SilentRestart = false              // Just shut down and restart

   // BDE
   this.BDErestart = true      // Restart BDE after update

   // Flag
   this.ChangeRequired = false // Does a setting need updating
   this.ValueChanged = false   // Has something been updated

   // Report
   this.ReportString = ""      // User report string
   
   // Runtime detection
   this.Runtime = "RUNTIME" $ upper(version(1))

   // Current directory
   if this.Runtime == true
      this.CurrDir = _dbwinhome.left(_dbwinhome.length -1) 
   else
      this.CurrDir = SET("DIRECTORY")
   endif

   // Error
   this.Error = 0
   this.LogErrors = false

   // Log File
   this.LogDir = ""            // Overide default location for log file
   this.RuntimeLogOnly = false // Log file only if in runtime

   //                                                            ************
   // BDE Settings start here <<---  <<---  <<---  <<---  <<---  BDE SETTINGS
   //                                                            ************

   // BDE_Reg_System_Init Section                     // BDE Installed Defaults
   this.AutoODBC = "TRUE"               // Important  // "FALSE"
   this.DataRepository = ""                           // ""
   this.DefaultDriver = "DBASE"                       // "PARADOX"
   this.LangDriver = "DBWINWE0"         // Important  // "'ascii' ANSI"
   this.LocalShare = "TRUE"             // Important  // false
   this.LowMemoryUsageLimit = "32"                    // "32"
   this.MaxBufSize = "16384"                          // "2048"
   this.MaxFileHandles = "4096"                       // "255"
   this.MemSize = "64"                                // "16"
   this.MinBufSize = "2048"                           // "128"
   this.MtsPooling = "FALSE"                          // "FALSE"
   this.SharedMemLocation = ""                        // ""
   this.SharedMemSize = "16348"                       // "2048"
   this.SqlQueryMode = "SERVER"         // Important  // ""

   // BDE_Reg_System_Formats_Date Section             // BDE Installed Defaults
   this.FourDigitYear = "TRUE"          // Important  // "FALSE"
   this.LeadingZeroD = "TRUE"                         // "TRUE"
   this.LeadingZeroM = "TRUE"                         // "TRUE"
   this.Mode = "1"                                    // "1"
   this.Separator = "/"                               // "/"
   this.YearBiased = "TRUE"                           // "TRUE"

   // BDE_Reg_System_Formats_Time Section             // BDE Installed Defaults
   this.AmString = "AM"                               // "AM"
   this.PmString = "PM"                               // "PM"
   this.Seconds = "TRUE"                              // "TRUE"
   this.MilSeconds = "FALSE"                          // "FALSE"
   this.TwelveHour = "FALSE"                          // "FALSE"

   // BDE_Reg_System_Formats_Number Section           // BDE Installed Defaults
   this.DecimalDigits = "2"                           // "2"
   this.DecimalSeparator = "."                        // "."
   this.ThousandsSeparator = ","                      // ","
   this.LeadingZeroN = "TRUE"                         // "TRUE"

   // DbaseNativeDriverUpdate()                       // BDE Installed Defaults
   this.dBASELangDriver = "DBWINWE0"    // Important  // "'ascii' ANSI"         
   this.dBASELevel = "7"                // Important  // "4"                   
   this.dBASEMDXBlockSize = "1024"      // Important  // "1024"                 
   this.dBASEMemoFileBlockSize = "1024" // Important  // "1024"                 

   // MSSQLNativeDriverUpdate()                       // BDE Installed Defaults
   this.MSSQLConnectTimeout = "60"                    // "60"
   this.MSSQLTimeout = "300"                          // "300"
   this.MSSQLTraceMode = "0"                          // "0"
   this.MSSQLMaxDbProcesses = "31"                    // "31"
   this.MSSQLApplicationName = ""                     // ""
   this.MSSQLBatchCount = "200"                       // "200"
   this.MSSQLBlobEditLogging = "FALSE"                // "FALSE"
   this.MSSQLBlobSize = "32"                          // "32"
   this.MSSQLBlobsToCache = "64"                      // "64"
   this.MSSQLDatabaseName = ""                        // ""
   this.MSSQLDateMode = "0"                           // "0"
   this.MSSQLEnableBCD = "FALSE"                      // "FALSE"
   this.MSSQLEnableSchemaCache = "FALSE"              // "FALSE"
   this.MSSQLHostName = ""                            // ""
   this.MSSQLLangDriver = "DBWINWE0"                  // "DBWINWE0"
   this.MSSQLMaxQueryTime = "300"                     // "300"
   this.MSSQLMaxRows = "-1"                           // "-1"
   this.MSSQLNationalLangName = ""                    // ""
   this.MSSQLOpenMode = "READ/WRITE"                  // "READ/WRITE"
   this.MSSQLSchemaCacheDir = ""                      // ""
   this.MSSQLSchemaCacheSize = "8"                    // "8"
   this.MSSQLSchemaCacheTime = "-1"                   // "-1"
   this.MSSQLServerName = ""                          // "MSS_SERVER"
   this.MSSQLSqlPassthruMode = "SHARED AUTOCOMMIT"    // "SHARED AUTOCOMMIT"
   this.MSSQLSqlQryMode = "SERVER"      // Important  // ""
   this.MSSQLTdsPacketSize = "4096"                   // "4096"
   this.MSSQLUserName = ""                            // "MYNAME"


   // -------------------------------------

   protect SetStringValue, SetNumericValue, HandleError

   protect AskTheUser, UserAgrees

   protect ChangeRequired, ValueChanged, ReportString, Runtime, CurrDir


   // -------------------------------------

   function SetStringValue(oReg, sValueName, sNewValue)

      if not this.UserAgrees == "No"

         local sOldValue

         sOldValue = oReg.GetValue(sValueName)

         // Check if update required first
         if not sOldValue == sNewValue

            // Update control property
            this.ChangeRequired := true 

            if this.AskUser == true and this.UserAgrees == "Unknown"
               if this.AskTheUser() == false 
                  return false
               endif
            endif

            // Do the update
            oReg.setValue(sValueName, sNewValue)

            // Trap error
            if oReg.Error # ERROR_SUCCESS
               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "    " + ERROR_TITLE + "   #" + oReg.Error + CR ;
                                  + "     " + ERROR_EXPLAIN + "   " + sValueName + CR 
               if oReg.Error == ERROR_ACCESS_DENIED
                  this.ReportString := this.ReportString ;
                                     + "     " + RIGHTS_EXPLAIN_1 + CR ;
                                     + "     " + RIGHTS_EXPLAIN_2 + CR 
               endif
               this.ReportString := this.ReportString + CR
               this.HandleError(sValueName, (oReg.Error))
               return false
            endif

            // Update control property
            this.ValueChanged := true

            if RMLANGUAGE == "German"
               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "     " + sValueName + " :-  " + TELL_SETTING + " " + TELL_FROM + " " + sOldValue + " " + TELL_TO + " " + sNewValue + " " + TELL_CHANGED + CR ;
                                  + CR
            else

               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "     " + sValueName + " :-  " + TELL_SETTING + " " + TELL_CHANGED + " " + TELL_FROM + " " + sOldValue + " " + TELL_TO + " " + sNewValue + CR ;
                                  + CR
            endif
          
         endif

      endif

   return true

   // -------------------------------------

   function SetNumericValue(oReg, sValueName, sNewValue)

      if not this.UserAgrees == "No"

         private sOldValue

         sOldValue = oReg.GetValue(sValueName)

         if type("sOldValue") == "L" //////////////////////////////////////////
            return false
         endif

         // Check if update required first

         // Use this line to make any numeric change
         if not val(sOldValue) == val(sNewValue)

         // Use this line to permit only the setting of higher values
         // if not val(sOldValue) >= val(sNewValue)

            // Update control property
            this.ChangeRequired := true

            if this.AskUser == true and this.UserAgrees == "Unknown"
               if this.AskTheUser() == false
                  return false
               endif
            endif

            // Do the update
            oReg.setValue(sValueName, sNewValue)

            // Trap error
            if oReg.Error # ERROR_SUCCESS
               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "    " + ERROR_TITLE + "   #" + oReg.Error + CR ;
                                  + "     " + ERROR_EXPLAIN + "   " + sValueName + CR 
               if oReg.Error == ERROR_ACCESS_DENIED
                  this.ReportString := this.ReportString ;
                                     + "     " + RIGHTS_EXPLAIN_1 + CR ;
                                     + "     " + RIGHTS_EXPLAIN_2 + CR 
               endif
               this.ReportString := this.ReportString + CR
               this.HandleError(sValueName, (oReg.Error))
               return false
            endif

            // Update control property
            this.ValueChanged := true

            if RMLANGUAGE == "German"
               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "     " + sValueName + " :-  " + TELL_SETTING + " " + TELL_FROM + " " + sOldValue + " " + TELL_TO + " " + sNewValue + " " + TELL_CHANGED + CR ;
                                  + CR
            else

               this.ReportString := this.ReportString ;
                                  + oReg.GetKeyName() + CR ;
                                  + "     " + sValueName + " :-  " + TELL_SETTING + " " + TELL_CHANGED + " " + TELL_FROM + " " + sOldValue + " " + TELL_TO + " " + sNewValue + CR ;
                                  + CR
            endif

         endif

      endif

   return true

   // -------------------------------------

   function Update_BDE_Init(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\SYSTEM\INIT

      // Instantiate registry object:
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\SYSTEM\INIT")

      // Set Registry values
      this.SetStringValue(oReg, "AUTO ODBC", this.AutoODBC)
      this.SetStringValue(oReg, "DATA REPOSITORY", this.DataRepository)
      this.SetStringValue(oReg, "DEFAULT DRIVER", this.DefaultDriver)
      this.SetStringValue(oReg, "LANGDRIVER", this.LangDriver)
      this.SetStringValue(oReg, "LOCAL SHARE", this.LocalShare)
      this.SetNumericValue(oReg, "LOW MEMORY USAGE LIMIT", this.LowMemoryUsageLimit)
      this.SetNumericValue(oReg, "MAXBUFSIZE", this.MaxBufSize)
      this.SetNumericValue(oReg, "MAXFILEHANDLES", this.MaxFileHandles)
      this.SetNumericValue(oReg, "MEMSIZE", this.MemSize)
      this.SetNumericValue(oReg, "MINBUFSIZE", this.MinBufSize)
      this.SetStringValue(oReg, "MTS POOLING", this.MtsPooling)
      this.SetStringValue(oReg, "SHAREDMEMLOCATION", this.SharedMemLocation)
      this.SetStringValue(oReg, "SHAREDMEMSIZE", this.SharedMemSize)
      this.SetStringValue(oReg, "SQLQRYMODE", this.SqlQueryMode)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Update_BDE_Formats_Date(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\SYSTEM\FORMATS\DATE

      // Instantiate registry object:
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\SYSTEM\FORMATS\DATE")

      // Set Registry values
      this.SetStringValue(oReg, "FOURDIGITYEAR", this.FourDigitYear)
      this.SetStringValue(oReg, "LEADINGZEROD", this.LeadingZeroD)
      this.SetStringValue(oReg, "LEADINGZEROM", this.LeadingZeroM)
      this.SetStringValue(oReg, "MODE", this.Mode)
      this.SetStringValue(oReg, "SEPARATOR", this.Separator)
      this.SetStringValue(oReg, "YEARBIASED", this.YearBiased)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Update_BDE_Formats_Time(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\SYSTEM\FORMATS\TIME

      // Instantiate registry object
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\SYSTEM\FORMATS\TIME")

      // Set Registry values
      this.SetStringValue(oReg, "AMSTRING", this.AmString)
      this.SetStringValue(oReg, "PMSTRING", this.PmString)
      this.SetStringValue(oReg, "SECONDS", this.Seconds)
      this.SetStringValue(oReg, "MILSECONDS", this.MilSeconds)
      this.SetStringValue(oReg, "TWELVEHOUR", this.TwelveHour)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Update_BDE_Formats_Number(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\SYSTEM\FORMATS\NUMBER

      // Instantiate registry object
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\SYSTEM\FORMATS\NUMBER")

      // Set Registry values
      this.SetStringValue(oReg, "DECIMALDIGITS", this.DecimalDigits)
      this.SetStringValue(oReg, "DECIMALSEPARATOR", this.DecimalSeparator)
      this.SetStringValue(oReg, "THOUSANDSEPARATOR", this.ThousandsSeparator)
      this.SetStringValue(oReg, "LEADINGZERON", this.LeadingZeroN)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Update_DbaseNativeDriver(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\DRIVERS\DBASE\INIT

      // Instantiate registry object                        
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\INIT")

      // Set Registry values
      this.SetStringValue(oReg, "LANGDRIVER", this.dBASELangDriver)
      release object oReg

      // Instantiate registry object
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\TABLE CREATE")
       
      // Set Registry values
      this.SetStringValue(oReg, "LEVEL", this.dBASELevel)
      this.SetNumericValue(oReg, "MDX BLOCK SIZE", this.dBASEMDXBlockSize)
      this.SetNumericValue(oReg, "MEMO FILE BLOCK SIZE", this.dBASEMemoFileBlockSize)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Update_MSSQLNativeDriver(bFinalise)

      // English language Reg path :
      // HKEY_LOCAL_MACHINE\SOFTWARE\Borland\Database Engine\Settings\DRIVERS\MSSQL\INIT

      // --- Instantiate registry object:
      local oReg
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\MSSQL\INIT")

      // Set Registry values
      this.SetStringValue(oReg, "CONNECT TIMEOUT", this.MSSQLConnectTimeout)
      this.SetStringValue(oReg, "TIMEOUT", this.MSSQLTimeout)
      this.SetStringValue(oReg, "TRACE MODE", this.MSSQLTraceMode)
      this.SetStringValue(oReg, "MAX DBPROCESSES", this.MSSQLMaxDbProcesses)
      release object oReg

      // Instantiate registry object
      oReg = new rmRegistry()
      oReg.SetMainKey(HKEY_LOCAL_MACHINE)
      oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\MSSQL\DB OPEN")

      // Set Registry values
      this.SetStringValue(oReg, "APPLICATION NAME", this.MSSQLApplicationName)
      this.SetStringValue(oReg, "BATCH COUNT", this.MSSQLBatchCount)
      this.SetStringValue(oReg, "BLOB EDIT LOGGING", this.MSSQLBlobEditLogging)
      this.SetStringValue(oReg, "BLOB SIZE", this.MSSQLBlobSize)
      this.SetStringValue(oReg, "BLOBS TO CACHE", this.MSSQLBlobsToCache)
      this.SetStringValue(oReg, "DATABASE NAME", this.MSSQLDatabaseName)
      this.SetStringValue(oReg, "DATE MODE", this.MSSQLDateMode)
      this.SetStringValue(oReg, "ENABLE BCD", this.MSSQLEnableBCD)
      this.SetStringValue(oReg, "ENABLE SCHEMA CACHE", this.MSSQLEnableSchemaCache)
      this.SetStringValue(oReg, "HOST NAME", this.MSSQLHostName)
      this.SetStringValue(oReg, "LANGDRIVER", this.MSSQLLangDriver)
      this.SetStringValue(oReg, "MAX QUERY TIME", this.MSSQLMaxQueryTime)
      this.SetStringValue(oReg, "MAX ROWS", this.MSSQLMaxRows)
      this.SetStringValue(oReg, "NATIONAL LANG NAME", this.MSSQLNationalLangName)
      this.SetStringValue(oReg, "OPEN MODE", this.MSSQLOpenMode)
      this.SetStringValue(oReg, "SCHEMA CACHE DIR", this.MSSQLSchemaCacheDir)
      this.SetStringValue(oReg, "SCHEMA CACHE SIZE", this.MSSQLSchemaCacheSize)
      this.SetStringValue(oReg, "SCHEMA CACHE TIME", this.MSSQLSchemaCacheTime)
      this.SetStringValue(oReg, "SERVER NAME", this.MSSQLServerName)
      this.SetStringValue(oReg, "SQLPASSTHRU MODE", this.MSSQLSqlPassthruMode)
      this.SetStringValue(oReg, "SQLQRYMODE", this.MSSQLSqlQryMode)
      this.SetStringValue(oReg, "TDS PACKET SIZE", this.MSSQLTdsPacketSize)
      this.SetStringValue(oReg, "USER NAME", this.MSSQLUserName)
      release object oReg

      if bFinalise == true
         this.Finalise()
      endif

   return

   // -------------------------------------

   function Finalise()

      local cText 
      
      // Deal with Error
      if this.Error # ERROR_SUCCESS

         // Produce Report
         cText = ERROR_EXPLAIN + CR ;
               + CR ;
               + this.ReportString 

         if this.TellUserError == true
            msgbox(cText, TELL_TITLE, 64 + 0)
         else
            ? "============================================"
            ?
            ? cText
            ? "============================================"
         endif

         // Log file
         if not (this.RuntimeLogOnly == true and this.Runtime == true )

            if this.LogErrors == true

               local f, cFile

               if this.LogDir == ""
                  cFile = this.CurrDir + "\rmBDEConfigLog.txt"
               else
                  cFile = this.LogDir + "\rmBDEConfigLog.txt"
               endif

               f = new File()
               if f.exists(cFile)
                 f.open(cFile, "A")
               else
                 f.create(cFile, "A")
               endif
               f.puts("============================================================")
               f.puts(new Date().toLocaleString())
               f.puts("")
               f.puts(cText)
               f.puts("")
               f.close()

            endif

         endif

      endif
     

      // Deal with update required but failed - insufficient user rights
      if this.ChangeRequired == true and this.Error == ERROR_ACCESS_DENIED

         if this.TellUserUpdateStillRequired == true 

            cText = this.StringToParagraph(ERROR_EXPLAIN + ".", 45) + CR ;
                  + CR ;
                  + this.StringToParagraph(RIGHTS_EXPLAIN_1, 45) + CR ; 
                  + CR ;
                  + this.StringToParagraph(RIGHTS_EXPLAIN_2, 45) + CR ;
                  + CR ;
                  + this.StringToParagraph(RIGHTS_EXPLAIN_3, 45) 

            msgbox(cText, ASK_TITLE, 64 + 0)

         endif

      endif


      // Deal with changes made
      if this.ValueChanged == true

        // Produce Report
         cText = TELL_EXPLAIN + CR ;
               + CR ;
               + this.ReportString 

         if this.TellUserChanged == true
            msgbox(cText, TELL_TITLE, 64 + 0)
         else
            ? "============================================"
            ?
            ? cText
            ? "============================================"
         endif

         // Log file
         if not (this.RuntimeLogOnly == true and this.Runtime == true)

            local f, cFile

            if this.LogDir == ""
               cFile = this.CurrDir + "\rmBDEConfigLog.txt"
            else
               cFile = this.LogDir + "\rmBDEConfigLog.txt"
            endif

            f = new File()
            if f.exists(cFile)
              f.open(cFile, "A")
            else
              f.create(cFile, "A")
            endif
            f.puts("============================================================")
            f.puts(new Date().toLocaleString())
            f.puts("")
            f.puts(cText)
            f.puts("")
            f.close()

         endif

         if this.BDErestart == true

            if this.SilentRestart == false

               // Restart BDE Restart App
               cText = this.StringToParagraph(RESTART_EXPLAIN_1, 45) + CR ;
                     + CR ;
                     + this.StringToParagraph(RESTART_EXPLAIN_2, 45) + CR ;
                     + CR ;
                     + this.StringToParagraph(RESTART_EXPLAIN_3, 45)

               msgbox(cText, RESTART_TITLE, 64 + 0)

               this.RestartBDE()

               quit

            else

               run(true,_app.EXEName)  

               quit

            endif

         endif

      else

         if this.TellUserNoChange == true
            msgbox(TELL_NO_CHANGE, TELL_TITLE, 64 + 0)
         else
            ? "============================================"
            ?
            ? TELL_NO_CHANGE
            ?
            ? "============================================"
         endif

      endif

      // Reset flags
      this.Reset()

   return

   // -------------------------------------

   function HandleError(cRegKey, nError)

      // Update Error code
      this.Error := nError

      if this.TellUserError == true

         local cText

         cText = this.StringToParagraph(ERROR_EXPLAIN + "  " + cRegKey, 45)

         // Insufficient rights
         if nError == ERROR_ACCESS_DENIED
            cText := cText ;
                   + CR + CR ;
                   + this.StringToParagraph(RIGHTS_EXPLAIN_1, 45) + CR ;
                   + CR ;
                   + this.StringToParagraph(RIGHTS_EXPLAIN_2, 45)  
         endif

         msgbox(cText, ERROR_TITLE, 16 + 0)

      endif

   return

   // -------------------------------------

   function AskTheUser()

      local cText

      cText = this.StringToParagraph(ASK_EXPLAIN, 45) + CR ;
            + CR ;
            + this.StringToParagraph(ASK_QUESTION, 45) + CR ;
            + CR ;
            + this.StringToParagraph(ASK_ADVICE, 45) 

      if msgbox(cText, ASK_TITLE, 64 + 4) == 6 // Yes
         this.UserAgrees := "Yes"
         return true
      else
         this.UserAgrees := "No"
         return false
      endif 

   return

   // -------------------------------------

   function UpdateAll()

      this.Update_BDE_Init(false)
      this.Update_BDE_Formats_Date(false)
      this.Update_BDE_Formats_Time(false)
      this.Update_BDE_Formats_Number(false)
      this.Update_DbaseNativeDriver(false)
      this.Update_MSSQLNativeDriver(false)
      
      this.Finalise()

      this.Reset()

   return

   // -------------------------------------

   function Reset()

      this.UserAgrees := "Unknown"
      this.ValueChanged := false
      this.ReportString := ""
      this.Error := 0
      this.ChangeRequired := false

   return

   // -------------------------------------

   function RestartBDE()

      // Code Vic McClung
      // Rewritten by Ronnie MacGregor to return boolean success

      local bSuccess

      bSuccess = false

      close databases

      if DbiExit() == DBIERR_NONE
         if DbiInitFn(300, null) == DBIERR_NONE
            bSuccess := true
         endif
      endif 

   return bSuccess

   // -------------------------------------

   function StringToParagraph(cString, cWidth)

      /*

      Ronnie MacGregor

      February 2006

      Takes a string of any length and returns the
      same string wrapped at the end of a whole word
      giving a line length less than or equal to cWidth

      */

      local aWords, aLines, cOutput

      cOutput = ""
      aWords = new array()
      aLines = new array()

      // Feed individual words into array
      do while len(cString) > 0
         // Chop off leading space
         do while substr(cString, 1, 1) == " "
            cString := substr(cString, 2)
         enddo
         // Add leading word to array
         if len(cString) > 0
            if " " $ cString
               aWords.add(substr(cString, 1, at(" ", cString, 1) - 1))
            else
               aWords.add(cString)
            endif
            cString := substr(cString, at(" ", cString, 1))
         endif
      enddo // next

      // Construct lines at less than paragraph width
      for nCount = 1 to aWords.size
         if len(cOutput) + 1 + len(aWords[nCount]) <= cWidth
            if len(cOutput) == 0
               cOutput := aWords[nCount]
            else
               cOutput := cOutput + " " + aWords[nCount]
            endif
         else
            aLines.add(cOutput)
            cOutput := aWords[nCount]
         endif
      endfor // next
      if len(cOutput) > 0
         aLines.add(cOutput)
      endif

      // Construct output paragraph
      cOutput = ""
      for nCount = 1 to aLines.size - 1
         cOutput := cOutput + aLines[nCount] + chr(13)
      endfor // next
      cOutput := cOutput + aLines[aLines.size]

   return cOutput

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof