//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------
/*
03/01/2017 - REM
Added the defines MAX_KEY_LENGTH and MAX_VALUE_NAME
Modified the function GetValueNameList()and GetSubKeyList()to use 
MAX_VALUE_NAME and MAX_KEY_LENGTH for string80 size 

*/



class rmRegistry

   this.Version = "2.08.0216"

   /*

   rmRegistry.cc 

   Windows 32 System Registry Class

   For         :  dBASE Plus / dB2K / Win9x / Win200x / WinXP



   Author      :  Repackaged by Ronnie MacGregor

   Contact     :  rmRegistryFeedback@dBASEdeveloper.co.uk

   Copyright   :  All Authors - All Rights Reserved.

   Disclaimer  :  It is the duty of the user of this code to determine it's 
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.
                  
                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.


   Acknowledgements :

                  Code written by :

                     dBASE Samples Group
                     Copyright (c) 1997-1998, Borland International, Inc.
                     Copyright (c) 2000, dBASE, Inc.
                     All rights reserved.

                  Additional code by :

                     Jim Sare
                     Tilman Rtger
                     Michael Nuwer
                     Ronnie MacGregor


   History and historical notes :

      Derived from Registry.prg  --  Windows 32 System Registry Class

      The registry program contains the class definition for the
      Registry class. Using this class you can read and write
      values in the Windows 32 registry.

      Version 1.0 

         Packaged with VdB 7.0
      

      Version 1.1 April 3, 2002

         Repair of SUBSTR call in EnumValue method


      Version 1.15 07/26/2003 Jim Sare
        
         Extend the Registry class to read values of type REG_MULTI_SZ.
         When calling oReg.GetValue, if the value is of any type
         currently supported by this class then that value will be
         returned. If the value is of type REG_MULTI_SZ, then an array
         of those values will be returned.


      Version 1.2 November 28, 2003

         setValue and GetValue methods now PROPERLY handle DWORDs.


      Version 1.21 April 11, 2005 Tilman Rtger  
                                 
         New Parameter ReadOnly: to read keys where you don't have      
         rights to write or change keys or values                       


      version 1.4 April 26, 2005 Michael Nuwer

         Added an Init() method which contains the control code from the 
         original class's constructor. This change makes it possible for
         a developer to override the Init() method with custom code.


      version 2.06.0301 March 1, 2006 Ronnie MacGregor

         Repackaged and code tidied up.
         Various typos and minor bugs corrected along the way.

         Now coded to work in an OO way only.

         Extensive renaming of properties variables etc. for clarity.

         Automatic readonly methods are now used unless required for
         creation, editing or deleting.

         Removal of the need for an Init method.

         New SetMainKey() method with boolean return of success.

         New SetSubKey() method with boolean return of success.

         New CreateKey() method with boolean return of success.
         CreateKey is designed to fail if the Key already exists.

         DeleteKey() rewritten to give a boolean return of success.

         Header files moved into cc to remove any file dependency.

         GetValue() rewritten to deal with all data types in one function.
         Returns the value, an array of values, or false the value name does
         not exist.

         DeleteValue() rewritten to give a boolean return of success.

         GetValueNameList() returns an array, or now returns false if it fails.

         GetSubKeyList() returns an array, or now returns false if it fails.

         Demo form with sample code provided.


      version 2.06.0307 March 7, 2006 Ronnie MacGregor

         New GetSubKeyList() method.

            Returns an array of subkey names, or returns false if it fails.


      version 2.08.0216 Feb 16, 2008 Ronnie MacGregor
               
         Fixes GetValue() typos in REG_MULTI_SZ section, found and reported by
         Andrew Shimmin.


      	


   Custom Properties :

      Error                 Contains the Windows error number if an error
                            occured during the last registry operation.
                            Contains 0 if no error occured.


   Methods :

      CreateKey()           Creates a key (only if it doesn't exist).

      DeleteKey()           Delete current key.

      DeleteValue([<name>]) Delete the named value from the current
                            key. If no name is passed, then the
                            default value for this key is deleted.

      GetValueNameList()    Returns an array containing the names of each
                            name value pair contained in the current key.

      GetKeyName()          Returns the name of the current registry key.

      GetSubKeyList()       Returns an array containing the name of each
                            subkey.

      GetValue(<name>)      Returns the value associated with <name>. The
                            <name> parameter is required, but may be blank.
                            If blank the default value for the key is
                            returned.

      SetMainKey(<n>)       Sets the main registry key for subsequent
                            operations.

      SetSubKey(<n>)        Sets the registry sub key for subsequent
                            operations.

      SetValue(<name>,<value>[,<type>])

                            Sets the value of <name> to <value>. Both
                            parameters are required, but <name> may be
                            blank to set the default value for the key.
                            Returns a logical true or false to indicate
                            success or failure, respectively. If no type
                            is indicated, the value is saved. The types
                            are defined in WINREG.H.



   Examples :

      Using the rmRegistry class to get the DBASE table creation level :

         #define HKEY_LOCAL_MACHINE (0x80000002)
         #define BDE_REG_KEY "SOFTWARE\Borland\Database Engine"

         local oReg
         oReg = new rmRegistry()
         oReg.SetMainKey(HKEY_LOCAL_MACHINE)
         oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\TABLE CREATE")
         ? oReg.GetValue("LEVEL")
         release object oReg



      Using the rmRegistry class to set the DBASE table creation level to 7 :

         #define HKEY_LOCAL_MACHINE (0x80000002)
         #define BDE_REG_KEY "SOFTWARE\Borland\Database Engine"

         local oReg
         oReg = new rmRegistry()
         oReg.SetMainKey(HKEY_LOCAL_MACHINE)
         oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\TABLE CREATE")
         oReg.SetValue("LEVEL", 7)
         release object oReg




      Using the rmRegistry class to set the DBASE table creation level to 7
      with error checking along the way :

         function SetLevel()

            #define HKEY_LOCAL_MACHINE (0x80000002)
            #define BDE_REG_KEY "SOFTWARE\Borland\Database Engine"

            local oReg
            oReg = new rmRegistry()
            if oReg.SetMainKey(HKEY_LOCAL_MACHINE) == false
               msgbox("Error setting main key")
               release object oReg
               return false
            endif
            if oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\TABLE CREATE") == false
               msgbox("Error setting sub key")
               release object oReg
               return false
            endif
            if oReg.SetValue("LEVEL", 7) == false
               msgbox("Error setting value")
               release object oReg
               return false
            endif
            release object oReg
            msgbox("Table level set OK")

         return true



      Using the rmRegistry class to set the DBASE table creation level to 7
      with overall error check (code from rmRegistry.wfm) :


         #define HKEY_LOCAL_MACHINE (0x80000002)
         #define BDE_REG_KEY "SOFTWARE\Borland\Database Engine"

         local oReg, bSuccess
         bSuccess = false
         oReg = new rmRegistry()
         if oReg.SetMainKey(HKEY_LOCAL_MACHINE) == true
            if oReg.SetSubKey(BDE_REG_KEY + "\Settings\DRIVERS\DBASE\TABLE CREATE") == true
               bSuccess := oReg.SetValue("LEVEL", form.EF_VALUE.value) 
               form.EF_Result.value := bSuccess
               if bSuccess == true
                  msgbox("BDE Table level set to " + form.EF_VALUE.value)
               endif
            endif
         endif
         release object oReg
         if bSuccess == false
            msgbox("Failed to set BDE table level")
         endif



   Note : Tilman Rtger

      Check your winreg.h file: 
      #define-Statements containing (READ_CONTRO)
      should be corrected to (READ_CONTROL)
          
         #define STANDARD_RIGHTS_READ             (READ_CONTROL)
         #define STANDARD_RIGHTS_WRITE            (READ_CONTROL)
         #define STANDARD_RIGHTS_EXECUTE          (READ_CONTROL)

      Alternativly use this bug workaround
      #define READ_CONTRO      (0x00020000)

      This control includes all required "header" constants
      with these corrections and other additions made. This note is here
      for information only. (Ronnie)
      

   */

   //--------------------------------------------------------------------
   //--------------------------------------------------------------------
   //--------------------------------------------------------------------

   //--------------------------------------------------------------
   //
   //  WINDEF.H 
   //
   //  Basic Windows Type Definitions  
   // 
   //  Visual dBASE Samples Group
   //
   //  $Revision:   1.4  $
   //
   //  Portions copyright (c) 1985-1995, Microsoft Corp. 
   //
   //--------------------------------------------------------------

   #ifndef _WINDEF_
   #define _WINDEF_

   //  These can be used to include only part of the 
   //  WIN32API.PRG file.
   //

   #define WIN32API_ALLSECTIONS         0xFFFFFFFF
   #define WIN32API_KERNEL              0x00000001
   #define WIN32API_USER                0x00000002
   #define WIN32API_GDI                 0x00000004
   #define WIN32API_ADVAPI              0x00000008
   #define WIN32API_SHELL               0x00000010
   #define WIN32API_INIFILE             0x00000020
   #define WIN32API_REGISTRY            0x00000040
   #define WIN32API_DIALOG              0x00000080
   #define WIN32API_NETWORK             0x00000100

   #define ULONG               CULONG
   #define PULONG              CPTR CULONG
   #define USHORT              CUSHORT
   #define PUSHORT             CPTR CSHORT
   #define UCHAR               CUCHAR
   #define PUCHAR              CPTR
   #define PSZ                 CSTRING

   #define MAX_PATH          260

   #define CFALSE              0
   #define CTRUE               1

   #define DWORD               CULONG
   #define BOOL                CLOGICAL
   #define BYTE                CUCHAR
   #define WORD                CUSHORT
   //#define FLOAT               CFLOAT   // Conflicts with FLOAT()

   #define PFLOAT              CPTR CFLOAT
   #define PBOOL               CPTR CLONG  // Should be CLOGICAL
   #define LPBOOL              CPTR CLONG  // Should be CLOGICAL
   #define PBYTE               CPTR
   #define LPBYTE              CPTR
   #define PINT                CPTR CINT
   #define LPINT               CPTR CINT
   #define PWORD               CPTR CUSHORT
   #define LPWORD              CPTR CUSHORT
   #define LPLONG              CPTR CLONG
   #define PDWORD              CPTR DWORD
   #define LPDWORD             CPTR DWORD
   #define PVOID               CPTR
   #define LPVOID              CPTR
   #define LPCVOID             CPTR

   // #define INT                 CINT   // Can't use INT, conflicts with INT()
   #define UINT                CUINT
   #define PUINT               CPTR CUINT


   //
   //  BUILD Version: 0093     Increment this if a change has global effects
   //
   // Copyright (c) 1990-1995  Microsoft Corporation
   //
   // Module Name:
   //
   //    winnt.h
   //
   // Abstract:
   //
   //    This module defines the 32-Bit Windows types and constants that are
   //    defined by NT, but exposed through the Win32 API.
   //
   //

   //
   // Basics
   //

   #define VOID       CVOID
   #define CHAR       CCHAR
   #define SHORT      CSHORT
   #define LONG       CLONG

   #define WCHAR      CSHORT
   #define PWCHAR     CPTR CSHORT
   #define LPWCH      CPTR CSHORT
   #define PWCH       CPTR CSHORT
   #define LPCWCH     CPTR CSHORT
   #define PCWCH      CPTR CSHORT
   #define NWPSTR     CPTR CSHORT
   #define LPWSTR     CPTR CSHORT
   #define PWSTR      CPTR CSHORT
   #define LPCWSTR    CPTR CSHORT
   #define PCWSTR     CPTR CSHORT

   //
   // ANSI (Multi-byte Character) types
   //
   #define PCHAR      CPTR
   #define LPCH       CPTR
   #define PCH        CPTR
   #define LPCCH      CPTR
   #define PCCH       CPTR
   #define NPSTR      CSTRING
   #define LPSTR      CSTRING
   #define PSTR       CSTRING
   #define LPCSTR     CSTRING
   #define PCSTR      CSTRING

   //
   // Neutral ANSI/UNICODE types and macros
   //

   #define PTCHAR       CPTR
   #define PTBYTE       CPTR
   #define LPTCH        LPSTR
   #define PTCH         LPSTR
   #define PTSTR        LPSTR
   #define LPTSTR       LPSTR
   #define LPCTSTR      LPCSTR

   #define PSHORT       CPTR CSHORT
   #define PLONG        CPTR CLONG
   #define HANDLE       CHANDLE
   #define PHANDLE      CPTR CUINT

   //#define CCHAR        CCHAR      
   #define LCID         DWORD
   #define PLCID        PDWORD
   #define LANGID       WORD

   #define LONGLONG     CDOUBLE
   #define DWORDLONG    CDOUBLE

   #define PLONGLONG    CPTR CDOUBLE
   #define PDWORDLONG   CPTR CDOUBLE

   #define USN          LONGLONG

   #define LARGE_INTEGER    CPTR
   #define PLARGE_INTEGER   CPTR
   #define ULARGE_INTEGER   CPTR
   #define PULARGE_INTEGER  CPTR

   #define LUID             LARGE_INTEGER
   #define PLUID            PLARGE_INTEGER

   #define SECURITY_INFORMATION  DWORD
   #define PSECURITY_INFORMATION PDWORD
   // End winnt.h

   // Types use for passing & returning polymorphic values 
   #define WPARAM              UINT
   #define LPARAM              LONG
   #define LRESULT             LONG

   //
   //  Cast macros added for Visual dBASE
   //
   #define CASTBYTE(a)         (mod(a,2^8))
   #define CASTWORD(a)         (mod(a,2^16))
   #define CASTLONG(a)         (mod(a,2^32))

   #define MAKEWORD(a, b)      (bitor(CASTBYTE(a),bitlshift(CASTBYTE(b), 8)))
   #define MAKELONG(a, b)      (bitor(CASTWORD(a),bitlshift(CASTWORD(b),16)))
   #define LOWORD(l)           (CASTWORD(l))
   #define HIWORD(l)           (bitrshift(CASTLONG(l),16))
   #define LOBYTE(w)           (CASTBYTE(w))
   #define HIBYTE(w)           (bitrshift(CASTWORD(w),8))

   // #define HWND             CHANDLE  // conflicts with dBASE hWnd property
   #define CHWND               CHANDLE  // substitute for standard HWND
   #define HHOOK               CHANDLE

   #define ATOM                WORD

   #define SPHANDLE            CPTR CUINT
   #define LPHANDLE            CPTR CUINT

   #define HGLOBAL             HANDLE
   #define HLOCAL              HANDLE
   //#define GLOBALHANDLE        HANDLE   // Conflicts with API function name since
   //#define LOCALHANDLE         HANDLE   // Visual dBASE is not case sensitive

   #define FARPROC             CPTR
   #define NEARPROC            CPTR

   #define HGDIOBJ             HANDLE

   #define HACCEL              HANDLE
   #define HBITMAP             HANDLE
   #define HBRUSH              HANDLE
   #define HCOLORSPACE         HANDLE
   #define HDC                 HANDLE
   #define HGLRC               HANDLE
   #define HDESK               HANDLE
   #define HENHMETAFILE        HANDLE
   #define HFONT               HANDLE
   #define HICON               HANDLE
   #define PHICON              CPTR
   #define HMENU               HANDLE
   #define HMETAFILE           HANDLE
   #define HINSTANCE           HANDLE
   #define HMODULE             HINSTANCE   // HMODULEs can be used in place of HINSTANCEs 
   #define HPALETTE            HANDLE
   #define HPEN                HANDLE
   #define HRGN                HANDLE
   #define HRSRC               HANDLE
   #define HSTR                HANDLE
   #define HTASK               HANDLE
   #define HWINSTA             HANDLE
   #define HKL                 HANDLE

   #define HFILE               CINT
   #define HCURSOR             HICON   // HICONs & HCURSORs are polymorphic 

   #define COLORREF            CULONG         // DWORD
   #define LPCOLORREF          CPTR CULONG

   //
   //  Added for Visual dBASE
   //
   #define ENUM                SHORT
   #define PSTRUCTURE          CPTR
   #define LPSTRUCTURE         CPTR
   #define va_list             CPTR
   //
   // From WINGDI.H
   //
   #define PDEVMODEA           LPSTRUCTURE
   #define LPDEVMODEA          LPSTRUCTURE
   #define PDEVMODE            LPSTRUCTURE
   #define LPDEVMODE           LPSTRUCTURE


   // OTHERS
   #define LPRECT              LPSTRUCTURE
   #define LPPOINT             LPSTRUCTURE

   #endif // _WINDEF_ 

   //--------------------------------------------------------------------
   //--------------------------------------------------------------------
   //--------------------------------------------------------------------

   //--------------------------------------------------------------
   //
   //  WINREG.H
   //
   //    This file contains the constant and type definitions 
   //    for the Windows 32-Bit Registry API. This is a subset
   //    of WINNT.H
   //
   //  Visual dBASE Samples Group
   //
   //  $Revision:   1.3  $
   //
   //  Portions copyright (c) 1991-1995, Microsoft Corp. 
   //
   //--------------------------------------------------------------

   #ifndef WINNT_H
   #ifndef _WINREG_
   #define _WINREG_

   #include "windef.h"

   //
   // Requested Key access mask type.
   //

   #define REGSAM DWORD

   //
   // Type definitions.
   //

   #define HKEY  HANDLE
   #define PHKEY PHANDLE

   //
   // Reserved Key Handles.
   //

   #define HKEY_CLASSES_ROOT           (0x80000000)
   #define HKEY_CURRENT_USER           (0x80000001)
   #define HKEY_LOCAL_MACHINE          (0x80000002)
   #define HKEY_USERS                  (0x80000003)
   #define HKEY_PERFORMANCE_DATA       (0x80000004)
   #define HKEY_CURRENT_CONFIG         (0x80000005)
   #define HKEY_DYN_DATA               (0x80000006)

   #define PROVIDER_KEEPS_VALUE_LENGTH 0x1

   #define PVALCONTEXT PSTRUCTURE

   #define PVALUE STRUCTURE
   #define PPVALUEA PSTRUCTURE

   #define QUERYHANDLER  NEARPROC
   #define PQUERYHANDLER FARPROC

   #define REG_PROVIDER STRUCTURE
   #define PPROVIDER    PSTRUCTURE

   #define VALENT  STRUCTURE
   #define PVALENT PSTRUCTURE

   // From winnt.h
   //
   //  The following are masks for the predefined standard access types
   //

   // #define DELETE                           (0x00010000) // conflicts with 
   //                                                       // DELETE command
   #define READ_CONTROL                     (0x00020000)
   #define WRITE_DAC                        (0x00040000)
   #define WRITE_OWNER                      (0x00080000)
   #define SYNCHRONIZE                      (0x00100000)

   #define STANDARD_RIGHTS_REQUIRED         (0x000F0000)

   #define STANDARD_RIGHTS_READ             (READ_CONTROL) // (READ_CONTRO)
   #define STANDARD_RIGHTS_WRITE            (READ_CONTROL) // (READ_CONTRO)
   #define STANDARD_RIGHTS_EXECUTE          (READ_CONTROL) // (READ_CONTRO)

   #define STANDARD_RIGHTS_ALL              (0x001F0000)

   #define SPECIFIC_RIGHTS_ALL              (0x0000FFFF)

   //
   // Registry Specific Access Rights.
   //

   #define KEY_QUERY_VALUE         (0x0001)
   #define KEY_SET_VALUE           (0x0002)
   #define KEY_CREATE_SUB_KEY      (0x0004)
   #define KEY_ENUMERATE_SUB_KEYS  (0x0008)
   #define KEY_NOTIFY              (0x0010)
   #define KEY_CREATE_LINK         (0x0020)

   #define KEY_READ                (bitand(bitor(bitor(bitor(STANDARD_RIGHTS_READ, KEY_QUERY_VALUE), KEY_ENUMERATE_SUB_KEYS), KEY_NOTIFY), BITNOT( SYNCHRONIZE ) ) )
   #define KEY_WRITE               (bitand(bitor(bitor(STANDARD_RIGHTS_WRITE, KEY_SET_VALUE), KEY_CREATE_SUB_KEY), BITNOT( SYNCHRONIZE) ) )
   #define KEY_EXECUTE             (bitand(KEY_READ, BITNOT( SYNCHRONIZE ) ) )
   #define KEY_ALL_ACCESS          (bitand(bitor(bitor(bitor(bitor(bitor(bitor(STANDARD_RIGHTS_ALL, KEY_QUERY_VALUE), KEY_SET_VALUE), KEY_CREATE_SUB_KEY), KEY_ENUMERATE_SUB_KEYS), KEY_NOTIFY), KEY_CREATE_LINK), BITNOT(SYNCHRONIZE)))

   //#define KEY_READ                ((STANDARD_RIGHTS_READ | KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS | KEY_NOTIFY) & (~SYNCHRONIZE))
   //#define KEY_WRITE               ((STANDARD_RIGHTS_WRITE | KEY_SET_VALUE | KEY_CREATE_SUB_KEY) & (~SYNCHRONIZE))
   //#define KEY_EXECUTE             ((KEY_READ) & (~SYNCHRONIZE))
   //#define KEY_ALL_ACCESS          BITXOR( BITOR( BITOR( BITOR( BITOR( BITOR( BITOR( STANDARD_RIGHTS_ALL, KEY_QUERY_VALUE ), KEY_SET_VALUE ), KEY_CREATE_SUB_KEY ), KEY_ENUMERATE_SUB_KEYS ), KEY_NOTIFY ), KEY_CREATE_LINK ), SYNCHRONIZE)

   //
   // Open/Create Options
   //

   #define REG_OPTION_RESERVED         (0x00000000)   /* Parameter is reserved */
   #define REG_OPTION_NON_VOLATILE     (0x00000000)   /* Key is preserved */
                                                      /* when system is rebooted */
   #define REG_OPTION_VOLATILE         (0x00000001)   /* Key is not preserved */
                                                      /* when system is rebooted */
   #define REG_OPTION_CREATE_LINK      (0x00000002)   /* Created key is a */
                                                      /* symbolic link */
   #define REG_OPTION_BACKUP_RESTORE   (0x00000004)   /* open for backup or restore */
                                                      /* special access rules */
                                                      /* privilege required */

   #define REG_LEGAL_OPTION            (bitor(bitor(bitor(bitor(REG_OPTION_RESERVED, REG_OPTION_NON_VOLATILE), REG_OPTION_VOLATILE), REG_OPTION_CREATE_LINK), REG_OPTION_BACKUP_RESTORE))

   //
   // Key creation/open disposition
   //

   #define REG_CREATED_NEW_KEY         (0x00000001)   // New Registry Key created
   #define REG_OPENED_EXISTING_KEY     (0x00000002)   // Existing Key opened

   //
   // Predefined Value Types.
   //

   #define REG_NONE                    (0)   // No value type
   #define REG_SZ                      (1)   // Unicode nul terminated string
   #define REG_EXPAND_SZ               (2)   // Unicode nul terminated string
                                               // (with environment variable references)
   #define REG_BINARY                  (3)   // Free form binary
   #define REG_DWORD                   (4)   // 32-bit number
   #define REG_DWORD_LITTLE_ENDIAN     (4)   // 32-bit number (same as REG_DWORD)
   #define REG_DWORD_BIG_ENDIAN        (5)   // 32-bit number
   #define REG_LINK                    (6)   // Symbolic Link (unicode)
   #define REG_MULTI_SZ                (7)   // Multiple Unicode strings
   #define REG_RESOURCE_LIST           (8)   // Resource list in the resource map
   #define REG_FULL_RESOURCE_DESCRIPTOR (9)  // Resource list in the hardware description
   #define REG_RESOURCE_REQUIREMENTS_LIST (10)

   // From winerror.h
   #define ERROR_SUCCESS                      0
   #define ERROR_ACCESS_DENIED                5
   #define ERROR_MORE_DATA                  234 
   #define ERROR_NO_MORE_ITEMS              259


   #endif // _WINREG_
   #endif // WINNT_H
	
	#define MAX_KEY_LENGTH 255
	#define MAX_VALUE_NAME 16383

	
	

   //--------------------------------------------------------------------
   //--------------------------------------------------------------------
   //--------------------------------------------------------------------

   // Prepare API calls
   class::PrototypeAPI()

   // -------------------------------------

   // Custom Properties
   this.KeyHandle = 0
   this.MainKey = 0
   this.SubKey = ""
   this.KeyOpen = false   
   this.ReadOnly = true    
   this.Error = 0

   // Prevent external access to internal methods
   protect PrototypeAPI, OpenKey, CloseKey

   // Prevent external access to internal properties
   protect KeyHandle, MainKey, SubKey, KeyOpen, ReadOnly

   // -------------------------------------

   function PrototypeAPI()

      if type("RegCloseKey") # "FP"
         extern CLONG RegCloseKey(;
            HKEY ;
            );
            ADVAPI32
      endif

      if type("RegCreateKeyEx") # "FP"
         extern CLONG RegCreateKeyEx(;
             HKEY, ;                    // hKey
             LPCTSTR, ;                 // lpSubKey
             DWORD, ;                   // Reserved
             LPTSTR, ;                  // lpClass
             DWORD, ;                   // dwOptions
             REGSAM, ;                  // samDesired
             LPSTRUCTURE, ;             // lpSecurityAttributes
             PHKEY, ;                   // phkResult
             LPDWORD ;                  // lpdwDisposition
             ) ;
             ADVAPI32 ;
         from "RegCreateKeyExA"
      endif

      if type("RegDeleteKey") # "FP"
         extern CLONG RegDeleteKey(;
            HKEY, ;                     // hKey
            LPCSTR ;                    // lpSubKey
            );
            ADVAPI32 ;
         from "RegDeleteKeyA"
      endif

      if type("RegDeleteValue") # "FP"
         extern CLONG RegDeleteValue(;
            HKEY, ;                     // hKey
            LPTSTR ;                    // lpValueName
            );
            ADVAPI32 ;
         from "RegDeleteValueA"
      endif

      if type("RegEnumKeyEx") # "FP"
         extern CLONG RegEnumKeyEx(;
            HKEY, ;                     // hKey          0 for first call
            DWORD, ;                    // dwIndex
            LPSTR, ;                    // lpName
            LPDWORD, ;                  // lpcName
            LPDWORD, ;                  // lpReserved
            LPSTR, ;                    // lpClass
            LPDWORD, ;                  // lpcClass
            LPSTRUCTURE ;               // lpftLastTimeWrite
            );
            advapi32 ;
         from "RegEnumKeyExA"
      endif

      if type("RegEnumValue") # "FP"
         extern CLONG RegEnumValue(;
            HKEY, ;                     // hKey          0 for first call
            DWORD, ;                    // dwIndex
            LPTSTR, ;                   // lpValueName
            LPDWORD, ;                  // lpValueName
            DWORD, ;                    // lpReserved
            LPDWORD, ;                  // lpType
            LPBYTE, ;                   // lpData
            LPDWORD ;                   // lpcbData
            );
            ADVAPI32 ;
         from "RegEnumValueA"
      endif

      if type("RegOpenKeyEx") # "FP"
         extern CLONG RegOpenKeyEx( HKEY, LPCTSTR, DWORD, REGSAM, PHKEY ) ;
                      ADVAPI32 from "RegOpenKeyExA"
      endif


      if type("RegQueryDWORDValueEx") # "FP"
         extern CLONG RegQueryDWORDValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CPTR CLONG, ;
                      LPDWORD ) ADVAPI32 from "RegQueryValueExA"
      endif

      if type("RegQueryValueEx") # "FP"
         extern CLONG RegQueryValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, ;
                      LPDWORD ) ADVAPI32 from "RegQueryValueExA"
      endif

      if type("RegQueryValueExP5") # "FP"
         extern CLONG RegQueryValueExP5(HKEY, LPTSTR, DWORD, LPDWORD, CPTR, LPDWORD);
                      ADVAPI32 from "RegQueryValueExA"
      endIf

      if type("RegSetValueEx") # "FP"
         extern CLONG RegSetValueEx( HKEY, LPCTSTR, DWORD, DWORD, CSTRING, DWORD ) ;
                      ADVAPI32 from "RegSetValueExA"
      endif

      if type("RegSetDWORDValueEx") # "FP"
         extern CLONG RegSetDWORDValueEx( HKEY, LPCTSTR, DWORD, DWORD, CPTR CLONG, DWORD ) ;
                      ADVAPI32 from "RegSetValueExA"
      endif

      local bAsian
      bAsian = false
      #ifdef __asian__
         if type("RegQueryValueExChar") # "FP"
            extern CLONG RegQueryValueExChar( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, ;
                         LPDWORD ) ADVAPI32 from "RegQueryValueExA"
         endif
         if type("RegSetValueExChar") # "FP"
            extern CLONG RegSetValueExChar( HKEY, LPCTSTR, DWORD, DWORD, CSTRING, DWORD ) ;
                         ADVAPI32 from "RegSetValueExA"
         endif
         bAsian := true
      #endif

   return (bAsian)   

   // -------------------------------------

   function OpenKey()

      local nResult, nHandle

      // Initialise variable for Key Handle
      nHandle = 0     

      // Reset the Error property
      this.Error := 0

      // Open the Key
      if this.ReadOnly == true
         nResult = RegOpenKeyEx(this.MainKey, this.SubKey, 0, KEY_READ, nHandle)
      else                      
         nResult = RegOpenKeyEx(this.MainKey, this.SubKey, 0, KEY_ALL_ACCESS, nHandle)
      endif

      // Check for success
      if nResult == ERROR_SUCCESS
         this.KeyHandle := nHandle
         this.KeyOpen := true
         return true
      else
         this.Error := (nResult)
         return false
      endif

   return 

   // -------------------------------------

   function CloseKey()
      
      local nResult

      // Reset the Error property
      this.Error := 0

      // Close the Key
      nResult = RegCloseKey(this.KeyHandle)

      // Check for success
      if nResult == ERROR_SUCCESS
         this.KeyHandle := 0
         this.KeyOpen := false
         this.ReadOnly := true
         return true
      else
         this.Error := (nResult)
         return false
      endif

   return

   // -------------------------------------

   function SetMainKey(nMainKey)

      local bResult

      // Set property value
      this.MainKey := (nMainKey)

      // Test for valid key
      bResult = this.OpenKey()
      if bResult == true  
         this.CloseKey() 
      endif

   return bResult

   // -------------------------------------

   function SetSubKey(cSubKey)

      local bResult

      // Set property value
      this.SubKey := (cSubKey)

      // Test for valid key
      bResult = this.OpenKey()
      if bResult == true  
         this.CloseKey() 
      endif

   return bResult

   // -------------------------------------

   function GetKeyName()

      local KeyName

      KeyName = ""

      do case

         case (this.MainKey == HKEY_CLASSES_ROOT )
            KeyName := "HKEY_CLASSES_ROOT\\"

         case (this.MainKey == HKEY_CURRENT_USER )
            KeyName := "HKEY_CURRENT_USER\\"

         case (this.MainKey == HKEY_LOCAL_MACHINE )
            KeyName := "HKEY_LOCAL_MACHINE\\"

         case (this.MainKey == HKEY_USERS )
            KeyName := "HKEY_USERS\\"

         case (this.MainKey == HKEY_PERFORMANCE_DATA )
            KeyName := "HKEY_PERFORMANCE_DATA\\"

         case (this.MainKey == HKEY_CURRENT_CONFIG )
            KeyName := "HKEY_CURRENT_CONFIG\\"

         case (this.MainKey == HKEY_DYN_DATA )
            KeyName := "HKEY_DYN_DATA\\"

         otherwise
            KeyName := "UNKNOWN_KEY\\"

      endcase

   return (KeyName + this.SubKey)

   // -------------------------------------

   function CreateKey(cNewKey)

      local nResult, nKeyHandle, nDisposition

      nKeyHandle   =  0
      nDisposition = -1

      // Check for backslash
      if not substr(cNewKey, 1, 1) == "\"
         cNewKey = "\" + cNewKey
      endif

      // Create the Key
      nResult = RegCreateKeyEx(this.MainKey, ;
                   this.SubKey + cNewKey, ;
                   0, ;
                   0, ;
                   REG_OPTION_NON_VOLATILE, ;
                   KEY_ALL_ACCESS, ;
                   0, ;
                   nKeyHandle, ;
                   nDisposition ;
                   )

      // Check for success
      if nResult == ERROR_SUCCESS
         if nDisposition == REG_CREATED_NEW_KEY 
            this.KeyHandle := nKeyHandle
            this.CloseKey()
            return true
         else // nDisposition == REG_OPENED_EXISTING_KEY
            this.CloseKey()
            return false
         endif 
      else
         this.Error := nResult
         return false
      endif

   return 

   // -------------------------------------

   function DeleteKey()

      local nResult

      // Delete the Key
      nResult = RegDeleteKey(this.MainKey, this.SubKey)

      // Check for success
      if nResult == ERROR_SUCCESS
         this.KeyHandle := 0
         this.KeyOpen := false
         return true
      else
         return false
      endif

   return 

   // -------------------------------------

   function GetValue(cValueName)

      local nResult, nType, nLen, xValue, xData
      local cData, cString, nCount, aValues 

      // Reset the Error property
      this.Error := 0

      // Open the key
      this.OpenKey()

      if not this.KeyOpen == true
         return false
      endif

      // Query value just to get value type
      nType = 0
      cData = chr(0)
      nLen = 1
      nResult = RegQueryValueEx(this.KeyHandle, cValueName, 0, nType, cData, nLen)

      // Deal with different types
      if nType == REG_MULTI_SZ

         // Query value
         cData := replicate(chr(0), ceiling(nLen / 2))
         nResult := RegQueryValueExP5(this.KeyHandle, cValueName, 0, nType, cData, nlen)

         // Check for success
         if nResult == ERROR_SUCCESS
            aValues = new array()
            cString = ""
            for nCount = 0 to nLen - 1
               if cData.getByte(nCount) # 0
                  cString += chr(cData.getByte(nCount))
               elseif not empty(cString)
                  aValues.add(cString)
                  cString := ""
               endif
            endfor
            this.CloseKey()
            return aValues
         else
            this.Error := nResult
            this.CloseKey()
            return 
         endif

      else

         // Query value
         xData   = Replicate(" ", nLen)
         nResult := RegQueryValueEx(this.KeyHandle, cValueName, 0, nType, xData, nLen)

         #ifdef __asian__
         // The Asian version uses Unicode strings. Call RegQueryValueExChar, which
         // is prototyped to char*, which converts the string to multi-byte.
         if (nResult == ERROR_SUCCESS and nType == REG_SZ)
            nResult := RegQueryValueExChar(this.KeyHandle, cValueName, 0, nType, xData, nLen)
         endif
         #endif

         if nType == REG_DWORD // get the data as a number
            xData := 0
            nResult := RegQueryDWORDValueEx(this.KeyHandle, cValueName, 0, nType, xData, nLen)
         endif

         // Check for success
         if nResult == ERROR_SUCCESS
            if (nType # REG_DWORD)
               xValue = SUBSTR(xData, 1, nLen - 1)
            else
               xValue = xData
            endif
            this.CloseKey()
            return xValue
         else
            this.Error := nResult
            this.CloseKey()
            return false
         endif

      endif

   return 

   // -------------------------------------

   function SetValue(cValueName, xNewValue, Type)

      local bReturn, nType, xValue, nAtNull, nLen, nResult

      private typeVal

      bReturn = false
      nType   = iif(ArgCount() == 3, type, REG_SZ)
      typeVal = xNewValue
      xValue  = xNewValue
      nAtNull = 0
      nLen    = 0
      nResult = 0

      // Reset the error property
      this.Error := 0

      // Open the key
      this.ReadOnly := false 
      this.OpenKey()

      if this.KeyOpen == true

         // Reformat data if necessary
         if (nType == REG_DWORD)

            if type("typeVal") == "C"
               xValue := VAL(xNewValue)
            endif
            nLen := 4

         else

            xValue  := xNewValue + "" // force to string type
            nAtNull := AT( CHR(0), xValue )
            if ( nAtNull > 0 )
               xValue := SUBSTR( xValue, 1, nAtNull + 1 )
            else
               xValue := xValue + CHR(0)
            endif
            nLen := LEN( xValue )

         endif

         // Write the data to the registry
         if (nType == REG_DWORD)
            nResult := RegSetDWORDValueEx(this.KeyHandle, cValueName, ;
                          0, nType, xValue, nLen)
         else
            #ifdef __asian__
            if (nType == REG_SZ)

               nResult := RegSetValueExChar(this.KeyHandle, cValueName, ;
                             0, nType, xValue, nLen)
            else
            #endif
               nResult := RegSetValueEx(this.KeyHandle, cValueName, ;
                             0, nType, xValue, nLen)
            #ifdef __asian__
            endif
            #endif
         endif

         if (nResult == ERROR_SUCCESS)
            bReturn := true
         else
            this.Error = (nResult)
         endif

         this.CloseKey()
         this.ReadOnly := true

      endif

   return (bReturn)

   // -------------------------------------

   function DeleteValue(cValueName)

      // If no parameter is passed then delete default value
      if ArgCount() == 0
         cValueName = ""
      endif

      local nResult

      // Reset the error property
      this.Error := 0

      // Open up the key
      this.ReadOnly := false
      this.OpenKey()

      if not this.KeyOpen == true
         return false
      endif

      // Delete the Value
      nResult = RegDeleteValue(this.KeyHandle, cValueName)

      // Check for success
      if nResult == ERROR_SUCCESS
         this.CloseKey()
         return true
      else
         this.CloseKey()
         return false
      endif

   return

   // -------------------------------------

   function GetValueNameList()

      local aValues, nResult, nCount, sValue, nLen, string80

      aValues = new Array()
      nResult = ERROR_SUCCESS
      nCount = 0
      sValue = ""
      nLen = 0
      string80 = REPLICATE(" ",MAX_VALUE_NAME)

      // Reset the error property
      this.Error := 0

      // open up the key
      this.OpenKey()

      if this.KeyOpen == true

         do while nResult == ERROR_SUCCESS

            sValue  := string80
            nLen    := sValue.length
            nResult := RegEnumValue(this.KeyHandle, nCount, sValue, nLen, null, null, null, null)

            if nResult == ERROR_SUCCESS
               aValues.add(substr(sValue, 1, nLen))
            else
               if nResult == ERROR_NO_MORE_ITEMS
                  this.CloseKey()
                  aValues.sort()
                  return aValues
               else
                  this.Error := nResult
                  this.CloseKey()
                  return false
               endif
            endif

            nCount ++

         enddo

      endif

   return

   // -------------------------------------

   function GetSubKeyList()

      // Ronnie MacGregor 2006/03/07

      local aSubKeys, nResult, nCount, cSubKeyName, nLen, string80

      aSubKeys = new Array()
      nResult = ERROR_SUCCESS
      nCount = 0
      cSubKeyName = ""
      nLen = 0
      string80 = REPLICATE(" ", MAX_KEY_LENGTH)

      // Reset the error property
      this.Error := 0

      // open up the key
      this.OpenKey()

      if this.KeyOpen == true

         do while nResult == ERROR_SUCCESS

            cSubKeyName  := string80 //+ 1
            nLen    := cSubKeyName.length
            nResult := RegEnumKeyEx(this.KeyHandle, nCount, cSubKeyName, nLen, null, null, null, null)

            if nResult == ERROR_SUCCESS
               aSubKeys.add(substr(cSubKeyName, 1, nLen))
            else
               if nResult == ERROR_NO_MORE_ITEMS
                  this.CloseKey()
                  aSubKeys.sort()
                  return aSubKeys
               else
                  this.Error := nResult
                  this.CloseKey()
                  return false
               endif
            endif

            nCount ++

         enddo

      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof
